#
# FFTW_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the FFTW and C extension libraries. If an
# argument is provided the symbols are setup for building FFTW, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([FFTW_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBFFTW='-lfftw3 -lm'
    else
        LIBFFTW='$(top_builddir)/fftw3/libfftw3.la -lm'
    fi

   AC_SUBST(LIBFFTW)
])


# FFTW_CHECK_LIBS
#---------------
# Checks for the FFTW libraries and header files.
AC_DEFUN([AC_SEARCH_FFTW],
[

    AC_MSG_CHECKING([for FFTW])

    fftw_check_fftw_header="fftw3.h"
    fftw_check_fftw_lib="libfftw3.a"
    fftw_includes=""
    ccunt_libraries=""
    AC_ARG_WITH(fftw,
                AC_HELP_STRING([--with-fftw],
                               [location where FFTW is installed]),
                [
                    fftw_with_fftw_includes="$withval/include $withval"
                    fftw_with_fftw_libs="$withval/lib64 $withval/lib32 $withval/lib $withval"
                ])

    AC_ARG_WITH(fftw-includes,
                AC_HELP_STRING([--with-fftw-includes],
                               [location of the FFTW header files]),
                fftw_with_fftw_includes=$withval)

    AC_ARG_WITH(fftw-libs,
                AC_HELP_STRING([--with-fftw-libs],
                               [location of the FFTW library]),
                fftw_with_fftw_libs=$withval)

    AC_ARG_ENABLE(fftw-test,
                  AC_HELP_STRING([--disable-fftw-test],
                                 [disables checks for the FFTW library and headers]),
                  fftw_enable_fftw_test=$enableval,
                  fftw_enable_fftw_test=yes)


    if test "x$fftw_enable_fftw_test" = xyes; then

        # Check for the FFTW includes

        if test -z "$fftw_with_fftw_includes"; then
            fftw_incdirs="/opt/fftw/include \
                         /usr/local/include \
                         /usr/local/include \
                         /usr/include \
                         $HOME/sph3rd/include \
                         $FFTW_SEARCH_LOCAL/include \
                         /usr/local/include \
                         $FFTW_SEARCH_LOCAL/include/fftw"
        else
            fftw_incdirs="$fftw_with_fftw_includes"
        fi
        test -n "$FFTWDIR" && fftw_incdirs="$FFTWDIR/include $FFTWDIR $fftw_incdirs"
        ESO_FIND_FILE($fftw_check_fftw_header, $fftw_incdirs, fftw_includes)


        # Check for the FFTW libraries

        if test -z "$fftw_with_fftw_libs"; then
            fftw_libdirs="/opt/fftw/lib \
                         /usr/local/lib \
                         /usr/local/fftw/lib \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $FFTW_SEARCH_LOCAL/lib"
        else
            fftw_libdirs="$fftw_with_fftw_libs \
                         /opt/fftw/lib \
                         /usr/local/lib \
                         /usr/local/fftw/lib \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $FFTW_SEARCH_LOCAL/lib"
        fi
	test -n "$FFTWDIR" && fftw_libdirs="$FFTWDIR/lib $FFTWDIR $fftw_libdirs"
        ESO_FIND_FILE($fftw_check_fftw_lib, $fftw_libdirs, fftw_libraries)


        if test x"$fftw_includes" = xno || test x"$fftw_libraries" = xno; then
            fftw_notfound=""

            if test x"$fftw_includes" = xno; then
                if test x"$fftw_libraries" = xno; then
                    fftw_notfound="(headers and libraries)"
                else            
                    fftw_notfound="(headers)"
                fi
            else
                fftw_notfound="(libraries)"
            fi

            AC_MSG_ERROR([FFTW $fftw_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $fftw_libraries, headers $fftw_includes])
        fi

        # Set up the symbols

        FFTW_INCLUDES="-I$fftw_includes"
        FFTW_LDFLAGS="-L$fftw_libraries"
        FFTW_CFLAGS="-I$fftw_includes"
	FFTW_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([FFTW checks have been disabled! This package may not build!])
        FFTW_INCLUDES=""
        FFTW_LDFLAGS=""
        FFTW_CFLAGS=""
        LIBFFTW=""
    fi

    AC_SUBST(FFTW_INCLUDES)
    AC_SUBST(FFTW_LDFLAGS)
    AC_SUBST(FFTW_CFLAGS)
    AC_SUBST(LIBFFTW)

])

