#
# GSL_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the GSL and C extension libraries. If an
# argument is provided the symbols are setup for building GSL, if no
# argument is given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([GSL_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBGSL='-lgsl -lgslcblas'
    else
        LIBGSL='$(top_builddir)/gsl/libgsl.la'
    fi

   AC_SUBST(LIBGSL)
])


# GSL_CHECK_LIBS
#---------------
# Checks for the GSL libraries and header files.
AC_DEFUN([AC_SEARCH_GSL],
[

    AC_MSG_CHECKING([for GSL])

    gsl_check_gsl_header="gsl/gsl_version.h"
    gsl_check_gsl_lib="libgsl.a"
    gsl_includes=""
    ccunt_libraries=""
    AC_ARG_WITH(gsl,
                AC_HELP_STRING([--with-gsl],
                               [location where GSL is installed]),
                [
                    gsl_with_gsl_includes="$withval/include $withval/include/gsl $withval"
                    gsl_with_gsl_libs="$withval/lib64 $withval/lib32 $withval/lib $withval"
                ])

    AC_ARG_WITH(gsl-includes,
                AC_HELP_STRING([--with-gsl-includes],
                               [location of the GSL header files]),
                gsl_with_gsl_includes=$withval)

    AC_ARG_WITH(gsl-libs,
                AC_HELP_STRING([--with-gsl-libs],
                               [location of the GSL library]),
                gsl_with_gsl_libs=$withval)

    AC_ARG_WITH(extra-includes,
                AC_HELP_STRING([--with-extra-includes],
                               [location of the extra header files]),
                gsl_with_gsl_includes=$withval)

    AC_ARG_WITH(extra-libs,
                AC_HELP_STRING([--with-extra-libs],
                               [location of the extra library]),
                gsl_with_gsl_libs=$withval)

    AC_ARG_ENABLE(gsl-test,
                  AC_HELP_STRING([--disable-gsl-test],
                                 [disables checks for the GSL library and headers]),
                  gsl_enable_gsl_test=$enableval,
                  gsl_enable_gsl_test=yes)


    if test "x$gsl_enable_gsl_test" = xyes; then

        # Check for the GSL includes

        if test -z "$gsl_with_gsl_includes"; then
            gsl_incdirs="/opt/gsl/include \
                         /usr/local/include \
                         /usr/local/gsl/include \
                         /usr/local/include \
                         /usr/include \
                         /usr/include \
                         $HOME/sph3rd/include \
                         $HOME/gslP/include \
                         $GSL_SEARCH_LOCAL/include \
                         /opt/gsl/include/gsl \
                         /usr/local/include/gsl \
                         /usr/local/gsl/include/gsl \
                         /usr/local/include/gsl \
                         /usr/include/gsl \
                         /usr/include/gsl \
                         $HOME/sph3rd/include/gsl \
                         $HOME/gslP/include/gsl \
                         $GSL_SEARCH_LOCAL/include/gsl"
        else
            gsl_incdirs="$gsl_with_gsl_includes"
        fi

        test -n "$GSLDIR" && gsl_incdirs="$GSLDIR/include $GSLDIR/gsl/ $gsl_incdirs"
        ESO_FIND_FILE($gsl_check_gsl_header, $gsl_incdirs, gsl_includes)


        # Check for the GSL libraries

        if test -z "$gsl_with_gsl_libs"; then
            gsl_libdirs="/opt/gsl/lib \
        		 /opt/gsl/lib64 \
                         /usr/local/lib \
                         /usr/local/lib64 \
                         /usr/local/gsl/lib \
                         /usr/local/gsl/lib64 \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $HOME/gslP/lib \
                         $HOME/gslP/lib64 \
                         $GSL_SEARCH_LOCAL/lib \
                         $GSL_SEARCH_LOCAL/lib64"
        else
            gsl_libdirs="$gsl_with_gsl_libs \
                         /opt/gsl/lib \
                         /opt/gsl/lib64 \
                         /usr/local/lib \
                         /usr/local/lib64 \
                         /usr/local/gsl/lib \
                         /usr/local/gsl/lib64 \
                         /usr/lib \
                         /usr/lib64 \
                         $HOME/sph3rd/lib \
                         $HOME/sph3rd/lib64 \
                         $HOME/gslP/lib \
                         $HOME/gslP/lib64 \
                         $GSL_SEARCH_LOCAL/lib \
                         $GSL_SEARCH_LOCAL/lib64"
        fi
	test -n "$GSLDIR" && gsl_libdirs="$GSLDIR/lib $gsl_libdirs"
        ESO_FIND_FILE($gsl_check_gsl_lib, $gsl_libdirs, gsl_libraries)


        if test x"$gsl_includes" = xno || test x"$gsl_libraries" = xno; then
            gsl_notfound=""

            if test x"$gsl_includes" = xno; then
                if test x"$gsl_libraries" = xno; then
                    gsl_notfound="(headers and libraries)"
                else            
                    gsl_notfound="(headers)"
                fi
            else
                gsl_notfound="(libraries)"
            fi

            AC_MSG_ERROR([GSL $gsl_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $gsl_libraries, headers $gsl_includes])
        fi

        # Set up the symbols

        GSL_INCLUDES="-I$gsl_includes -I$gsl_includes/gsl"
        GSL_LDFLAGS="-L$gsl_libraries"
        GSL_CFLAGS="-I$gsl_includes -I$gsl_includes/gsl"
	GSL_CREATE_SYMBOLS
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([GSL checks have been disabled! This package may not build!])
        GSL_INCLUDES=""
        GSL_LDFLAGS=""
        GSL_CFLAGS=""
        LIBGSL=""
    fi

    AC_SUBST(GSL_INCLUDES)
    AC_SUBST(GSL_LDFLAGS)
    AC_SUBST(GSL_CFLAGS)
    AC_SUBST(LIBGSL)

])

