/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#include "sph_common_keywords.h"
#include "sph_ifs_keywords.h"
#include "sph_dataset.h"
#include "sph_dataset_stats.h"
#include "sph_ifs_atmospheric.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"
#include "sph_ifs_tags.h"
#include "sph_ldt.h"
#include "sph_framecombination.h"
#include "sph_framesorter.h"
#include "sph_keyword_manager.h"
#include "sph_create_super_flat.h"
#include "sph_filemanager.h"
#include "sph_ifs_science.h"
/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

extern sph_error_code SPH_IFS_ATMOSPHERIC_GENERAL;
extern sph_error_code SPH_IFS_ATMOSPHERIC_FRAMES_MISSING;


/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_ifs_atmospheric_run Run the Instrument Flat Recipe
 *
 * This module provides the algorithm inplementation for the creation of the
 * instrument flat recipe for IFS.
 *
 * @par Synopsis:
 * @code
 *   #include "sph_ifs_atmospheric.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/
cpl_error_code
sph_ifs_atmospheric_run( sph_ifs_atmospheric* self )
{
    sph_ldt*        result    = NULL;
    sph_ifs_lenslet_model*    lensmodel = NULL;
    cpl_propertylist*        plist = NULL;

    plist = sph_keyword_manager_load_properties(cpl_frame_get_filename(self->wave_calib_frame),0);
    lensmodel = sph_ifs_lenslet_model_new_from_propertylist(plist);

    SPH_RAISE_CPL_RESET
    if ( self->use_adi == 0 && self->dither == 0 ) {
        result = sph_ifs_science_reduce_no_dither_no_adi(
                self->raw_science_frameset,
                self->coll_alg,
                self->framecomb_parameterlist,
                lensmodel,
                self->master_dark_frame,NULL,
                self->wave_calib_frame,NULL,
                self->master_ifu_flat_frame,
                self->master_dff_long1_frame,
                self->master_dff_long2_frame,
                self->master_dff_long3_frame,
                self->master_dff_long4_frame,
                self->master_dff_longbb_frame,
                self->master_dff_short_frame );
    }
    else if ( self->use_adi == 0 && self->dither == 1 ) {
        result = sph_ifs_science_reduce_dither_no_adi(
                self->raw_science_frameset,
                self->coll_alg,
                self->framecomb_parameterlist,
                lensmodel,
                self->master_dark_frame,NULL,
                self->wave_calib_frame,NULL,
                self->master_ifu_flat_frame,
                self->master_dff_long1_frame,
                self->master_dff_long2_frame,
                self->master_dff_long3_frame,
                self->master_dff_long4_frame,
                self->master_dff_longbb_frame,
                self->master_dff_short_frame );
    }

    if ( result ) {
        sph_ldt_save_cube( result,
                self->atmospheric_filename,
                self->inframes,NULL,
                self->inparams,
                SPH_IFS_TAG_ATMOSPHERIC_CALIB,
                SPH_RECIPE_NAME_IFS_ATMOSPHERIC,
                SPH_PIPELINE_NAME_IFS, plist );
        sph_ldt_delete( result ); result = NULL;
    }
    return cpl_error_get_code();
}
/**@}*/

