/***********************************************************************************************/
/******** DANGER !! DANGER !! DANGER !! DO NOT EDIT FILE -- IT WAS GENERATED AUTOMATICALLY !!! */
/***********************************************************************************************/

/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#include "sph_common_keywords.h"
#include "sph_ird_keywords.h"
#include "sph_ird_tags.h"
#include "sph_ird_science_dbi.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"

/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

sph_error_code SPH_IRD_SCIENCE_DBI_GENERAL              = SPH_IRD_SCIENCE_DBI_ERR_START + 0;
sph_error_code SPH_IRD_SCIENCE_DBI_NO_VALID_DIT_LIST    = SPH_IRD_SCIENCE_DBI_ERR_START + 1;

sph_error_code SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING    = SPH_IRD_SCIENCE_DBI_ERR_START + 3;
sph_error_code SPH_IRD_SCIENCE_DBI_FRAMES_MISSING       = SPH_IRD_SCIENCE_DBI_ERR_START + 4;

/*-----------------------------------------------------------------------------
 Private function prototypes
 -----------------------------------------------------------------------------*/

static int sph_ird_science_dbi_create_plugin ( cpl_plugin * );
static int sph_ird_science_dbi_exec_plugin ( cpl_plugin * );
static int sph_ird_science_dbi_destroy_plugin ( cpl_plugin * );

/*-----------------------------------------------------------------------------
 Static variables
 -----------------------------------------------------------------------------*/

static const char* sph_ird_science_dbi_help = 
    "RECIPE INPUT FRAMES: \n"
    "-------------------- \n"
    "Tag                           Min,Max   Description\n"
    "IRD_SCIENCE_DBI_RAW             1,500   The raw science frames.\n"
    "IRD_MASTER_DARK                 1,  1   The master dark frame. Should have the same DIT and readout mode\n"
    "                                        as the raw frames.\n"
    "IRD_FLAT_FIELD                  1,  1   The instrument flat. Should have the same filter set as the\n"
    "                                        raw frames.\n"
    "IRD_DISTORTION_MAP              0,  1   The distortion map frame.\n"
    "IRD_FILTER_TABLE                0,  1   A filter table, used to determine SDI scaling.\n"
    "                                        This is a static calibration frame of an ASCII table defining the IRDIS filters. It is\n"
    "                                        only used if SDI is selected.\n"
    "                                        The format of this table is:\n"
    "                                        <filter name>         <central wavelength total or left FOV (microns)>   <central wavelength right or 0>\n"
    "                                        with one line for every filter. The first line is treated as the default filter in case that there are\n"
    "                                        insufficient keywords to determine the filtername in the RAW input frames.\n"
    "IRD_STAR_CENTER                 0,  1   The field center frame. This frame must be as produced by the\n"
    "                                        star center recipe. If none are provided the recipe attempts\n"
    "                                        a fallback solution or makes use of the optional ASCII field center tables if provided.\n"
    "IRD_FCTABLE                     0, -1   Alternative input of set of of files of ASCII tables. The tables should each give one row\n"
    "                                        for each input science image (so, if e.g.  5 cubes of each 10 planes are this\n"
    "                                        frameset should contain 5 tables with 10 rows) with the columns centre-x centre-y centre-x centre-y angle\n"
    "                                        where centers for left and right field of view are given in pixel coordinates inside\n"
    "                                        the left/right subframe.\n"
    "IRD_STATIC_BADPIXELMAP          0,  1   An optional static badpixelmap. If a dark is given the badpixels\n"
    "                                        that are masked out before the fit are all pixels marked as bad\n"
    "                                        in this as well as in the dark frame. If no dark is given pixels\n"
    "                                        marked in this frame are marked as bad in the raw frames.\n"
    "\n"
    "RAW FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Type   Optional  Description\n"
    "ESO DRS IRD DUAL FILTER LAMBDA LEFTdouble Yes       The central wavelength of the filter on left. Only needed if SDI requested. \n"
    "ESO DRS IRD DUAL FILTER LAMBDA RIGHTdouble Yes       The central wavelength of the filter on right. Only needed if SDI requested. \n"
    "\n"
    "DESCRIPTION:\n"
    "------------\n"
    "This recipe creates the reduced  science frames for \n"
    "all science observations with IRDIS in DBI maging mode. The recipe\n"
    "supports dithered frame combination, as well as ADI and SDI\n"
    "The frames are reduced in the following steps:\n"
    "\ltx\n"
    "\begin{enumerate} \n"
    "\item The input raw frames are dark subtracted, if a dark is provided\n"
    "\item a flat field is divided out if it is provided\n"
    "\item a badpixel map is created for each frame that contains the union of all dark and flat\n"
    " field badpixels\n"
    "\item the left and right IRDIS subframes are extracted using the IRDIS instrument model\n"
    "as specified in the header of the flat field (if provided) or the default model otherwise.\n"
    "\end{enumerate}\n"
    "Now, for each of the subframes the processing is as follows:\n"
    "\begin{enumerate}\n"
    "\item high frequency filtering. If the filter radius $f_r$ is set to a value larger than 0, a top\n"
    "hat frequency filter is applied, masking out all frequencies above the value of $f > f_r \times f_{max}$,\n"
    "where $f_{max}$ is the maximum frequency in the FFT. For noise filterings a value between 0.9 and 0.99 are\n"
    "good values to use for $f_r$.\n"
    "\item FFT or warp shifting of image to recenter the image\n"
    "\item Application of the distortion map to image\n"
    "\item Shifting and distortion map application to badpixel map using geometrical approach\n"
    "\end{enumerate}\n"
    "All these processed frames are then saved as temporary files. These are then combined to create a reference speckle image.\n"
    "Now the ADI or SDI steps are performed if one of them or both are selected. If not is selected, these steps are skipped.\n"
    "\begin{enumerate}\n"
    "\item The speckle frame is subtracted.\n"
    "\item if SDI is selected, scaling of the images using FFT around the image center -- the angle should be given as part of the\n"
    "field center table provided in the input of the recipe.\n"
    "\item if ADI is selected, rotation of the images using FFT around the image center -- the angle should be given as part of the\n"
    "field center table provided in the input of the recipe.\n"
    "\end{enumerate}\n"
    "Then these frames are combined using the\n"
    "selected combination algorithm.\n"
    "\begin{itemize}\n"
    "\item if a weighted mean is selected, a weightmap is calculated first taking the median frame as a reference frame and\n"
    "weighing down the other frames depending on the difference in values. Note that this is still a very experimental option and\n"
    "it is still to be defined what weighting sheme would be optimal. All badpixels get assigned the weight of 0. Frames are then\n"
    "combined taking the individual weights into account.\n"
    "\item if a mean is selected, frames are combined using a mean, after first rejecting all bad pixels.\n"
    "\item if a median is selected, frames are combined by taking the median pixel value at each pixel position. This procedure\n"
    "ignores the badpixels.\n"
    "\end{itemize}\n"
    "The obtained combined results for the left and right IRDIS field of view are saved in a single FITS file with 8 extensions, following\n"
    "the layout for a double master frame: the first four extensions being the image, badpixelmap, N map, and rms for the left field and\n"
    "the second set of four extensions being the equivalent for the right field.\n"
    "\n"
    "Some additional notes:\n"
    "\begin{itemize}\n"
    "\item The static badpixel frame is optional and the badpixels defined there will be combined (using a logical OR) with badpixels in\n"
    "the dark or flat.\n"
    "\item Before the images are transformed (rotated and/or shifted) badpixels are interpolated. \n"
    "Interpolation happens irrespective of algorithm choice.\n"
    "The interpolation is a simple 8 neighbbour pixel average. In\n"
    "case a number of n < 8 neighbour pixels are also bad, 8 - n values are\n"
    "used. In case all neighbour pixels are bad, the interpolation simply copies the \n"
    "value from nearest non bad pixel.\n"
    "\item While a filter table is not strictly required, no scaling will be done if SDI is selected, leading to zero images.\n"
    "\end{itemize}\n"
    "\n"
    "\eltx\n"
    "RECIPE PRODUCTS: \n"
    "-------------------- \n"
    "Tag                      Format         Description\n"
    "IRD_SCIENCE_DBI          FITS[Im(8)]    The main science frame. The FITS file contains 8 extensions: the image, the\n"
    "                                        badpixels, the rms error and a weightmap for left and right field of views.\n"
    "\n"
    "PRODUCT FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Frame          Type    Description\n"
    "\n"
;

/*-----------------------------------------------------------------------------
 Function code
 -----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_ird_science_dbi Create Master Dark Recipe
 *
 * This module provides the recipe plugin and relating functions for the
 * creation of the master dark .
 *
 * @par Synopsis:
 * @code
 *   #include "sph_ird_science_dbi.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*****************************************************************************
 *
 *
 *      First come the static and global functions necessary for the
 *      normal working of the recipe as plugin....
 *
 *
 * ***************************************************************************/
/*----------------------------------------------------------------------------*/
/**
 @brief    Build the list of available plugins, for this module.
 @param    list    the plugin list
 @return   0 if everything is ok, 1 otherwise
 @note     Only this function is exported

 Create the recipe instance and make it available to the application
 using the interface.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe * recipe = cpl_calloc( 1, sizeof *recipe);
    cpl_plugin * plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                        CPL_PLUGIN_API,
                        SPH_IRD_SCIENCE_DBI_VERSION,
                        CPL_PLUGIN_TYPE_RECIPE,
                        "sph_ird_science_dbi",
                        "Science calibration, DBI mode.",
                        sph_ird_science_dbi_help, "Ole Moeller-Nilsson",
                        "moeller@mpia-hd.mpg.de", cpl_get_license("SPHERE DRH","2012"),
                        sph_ird_science_dbi_create_plugin,
                        sph_ird_science_dbi_exec_plugin,
                        sph_ird_science_dbi_destroy_plugin) )
    {
        cpl_msg_error( cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }

    if (cpl_pluginlist_append(list, plugin) )
    {
        cpl_msg_error( cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Setup the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int sph_ird_science_dbi_create_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe             = NULL;

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: An error is already set: %s",
                       cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error(cpl_func, "Null plugin");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    if ( plugin != NULL ) {
        recipe = (cpl_recipe *)plugin;
        recipe->parameters = sph_ird_science_dbi_create_paramlist();
        if ( recipe->parameters == NULL ) {
            return SPH_IRD_SCIENCE_DBI_GENERAL;
        }
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Testing for setup of the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Wrapper to make the static plugin creation function accessible from outside
 for testing purposes.
 */
/*----------------------------------------------------------------------------*/
int sph_ird_science_dbi_create_test(cpl_plugin * plugin)
{
    return (int)sph_ird_science_dbi_create_plugin(plugin);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Execute the plugin instance given by the interface
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_ird_science_dbi_exec_plugin(cpl_plugin * plugin)
{

    cpl_recipe*             recipe                  = NULL;
    int                     recipe_status           = 0;
    sph_ird_science_dbi*    sph_recipe      = NULL;
    cpl_errorstate          initial_errorstate      = cpl_errorstate_get();

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: A cpl error is already set: %s",
        cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
        return (int)cpl_error_get_code();
    }

    recipe = (cpl_recipe *)plugin;

    if (recipe->parameters == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (recipe->frames == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL frame set");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    sph_recipe = sph_ird_science_dbi_new( recipe->frames, recipe->parameters );
    if ( sph_recipe == NULL ) {
        cpl_msg_error( cpl_func, "SPHERE recipe initialisation call failed. "
                "Please check the SPHERE error messages.");
        sph_end_erex(); //close the sphere system
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    recipe_status = sph_ird_science_dbi_run( sph_recipe );

    sph_ird_science_dbi_delete( sph_recipe );

    if ( !cpl_errorstate_is_equal(initial_errorstate) )
    {
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }
    sph_end_erex(); //close the sphere system
    return recipe_status;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Destroy what has been created by the 'create' function
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_ird_science_dbi_destroy_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe;

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    recipe = (cpl_recipe *)plugin;

    cpl_parameterlist_delete(recipe->parameters);

    return 0;
}

/*****************************************************************************
 *
 *
 *      Here come the "Memnber" Functions of the sph_ird_science_dbi struct
 *
 *
 * ***************************************************************************/

/*----------------------------------------------------------------------------*/
/**
 @brief    Create and return the standard parameterlist for the recipe

 @return   the standard parameterlis or NULL if an error occcured.

 This function creates the standard parameterlist for the recipe
 and returns it.

 The return value is NULL when an error occured.
 */
/*----------------------------------------------------------------------------*/
cpl_parameterlist*  sph_ird_science_dbi_create_paramlist(void)
{
    cpl_parameterlist*          result              = NULL;
    cpl_parameter*              p                   = NULL;
    result = cpl_parameterlist_new();

    if (result == NULL)
    {
        cpl_msg_error( cpl_func, "Null parameterlist");
        return NULL;
    }
    else {
        /* Code to set up parameters GENERATED DO NOT EDIT */

        p = cpl_parameter_new_value("ird.science_dbi.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ird.science_dbi",
                                   "science_dbi.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.outfilename_left",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ird.science_dbi",
                                   "science_dbi_left.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.make_template",
                                   CPL_TYPE_BOOL,
                                   "if set to TRUE the recipe creates an empty template of the field center table to be filled by hand. "
                                   ,"ird.science_dbi",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.keep_fctable",
                                   CPL_TYPE_BOOL,
                                   "if set to TRUE the recipes internall created field center tables are not deleted. "
                                   ,"ird.science_dbi",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.outfilename_right",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"ird.science_dbi",
                                   "science_dbi_right.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("ird.science_dbi.coll_alg",
                                   CPL_TYPE_INT,
                                   "The collapse algorithm to use. 0 = Mean, 1 = Median. "
                                   ,"ird.science_dbi",
                                   1 ,
                                   2,0,1);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.use_adi",
                                   CPL_TYPE_BOOL,
                                   "Flag to control usage of ADI "
                                   ,"ird.science_dbi",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.use_sdi",
                                   CPL_TYPE_BOOL,
                                   "Flag to control usage of SDI "
                                   ,"ird.science_dbi",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.minr",
                                   CPL_TYPE_DOUBLE,
                                   "The minimum radius of the annulus used to renormalise the flux for SDI. "
                                   ,"ird.science_dbi",
                                   4.0 ,
                                   0.0,512.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.maxr",
                                   CPL_TYPE_DOUBLE,
                                   "The maximum radius of the annulus used to renormalise the flux for SDI. "
                                   ,"ird.science_dbi",
                                   40.0 ,
                                   0.0,512.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("ird.science_dbi.full_frameset_speck",
                                   CPL_TYPE_BOOL,
                                   "This sets whether speckle frames should be calculated per cube (if set to FALSE) or for "
                                   "the full set of frames (TRUE, default) "
                                   ,"ird.science_dbi",
                                   1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("ird.science_dbi.transform_method",
                                   CPL_TYPE_INT,
                                   "Transform method to use. 0 is FFT, 1 is CPL_WARP (interpolation). "
                                   ,"ird.science_dbi",
                                   0 ,
                                   2,0,1);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("ird.science_dbi.filter_method",
                                   CPL_TYPE_INT,
                                   "FFT filter method to use. 0 is none, 1 is top hat filter, 2 is Fermi fitler, 3 is Butterworth filter. "
                                   ,"ird.science_dbi",
                                   0 ,
                                   4,0,1,2,3);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.filter_rad",
                                   CPL_TYPE_DOUBLE,
                                   "Radius for FFT top hat and Fermi filters. "
                                   "A non zero value leads to suppression "
                                   "of high frequencies in the fourier domain before framecombination. The value "
                                   "expresses the minimum unsuppressed frequency as fraction of total frequency "
                                   "domain radius (a value of 1 would suppress essentially all frequencies). "
                                   ,"ird.science_dbi",
                                   0.0 ,
                                   0.0,1.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.fermi_temp",
                                   CPL_TYPE_DOUBLE,
                                   "The temperature parameter for the Fermi filter. "
                                   ,"ird.science_dbi",
                                   0.0 ,
                                   0.0,1.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.butter_pass",
                                   CPL_TYPE_DOUBLE,
                                   "The pass band frequency for the Butterworth filter, as fraction of total frequency domain radius. "
                                   ,"ird.science_dbi",
                                   0.0 ,
                                   0.0,1.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.butter_stop",
                                   CPL_TYPE_DOUBLE,
                                   "The stop band frequency for the Butterworth filter, as fraction of total frequency domain radius. "
                                   "This must be larger than the pass frequency. "
                                   ,"ird.science_dbi",
                                   0.0 ,
                                   0.0,1.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("ird.science_dbi.window_size",
                                   CPL_TYPE_INT,
                                   "When set to a non zero value, the recipe uses a special subwindow mode, where only "
                                   "cut-outs are of the given size are used (the cut out is made after dark and flat have "
                                   "been applied and the subfields have been extracted). For example to use only the central "
                                   "128 pixels for both left and right subfields use window_size=128. "
                                   ,"ird.science_dbi",
                                   0 ,
                                   0,1024);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        /* End of code to set up parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    }


    return result;


}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe frames, and assign them to the sph_ird_science_dbi
           structure.
 @param    self     the sph_ird_science_dbi recipe structure
 @return   error code of the check.

 This checks the frames that were given in the sph_ird_science_dbi_new function
 as parameters for the correctness and completeness. If the frames needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code  sph_ird_science_dbi_check_frames( sph_ird_science_dbi* self )
{
    sph_error_code           rerr            = CPL_ERROR_NONE;
    int                      ii	             = 0;
    cpl_frame*               aframe          = NULL;
    cpl_propertylist*        pl              = NULL;
    int	                     haskey          = 0;
    int                      okval           = 0;
    /* Code to check frames GENERATED DO NOT EDIT */

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_SCIENCE_DBI_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->rawframes = sph_utils_extract_frames( self->inframes, SPH_IRD_TAG_SCIENCE_DBI_RAW );
    if ( ! self->rawframes )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract rawframes frames."
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_SCIENCE_DBI_RAW);
        rerr = SPH_IRD_SCIENCE_DBI_FRAMES_MISSING;
        return rerr;
    }

    else {

         for ( ii = 0; ii < cpl_frameset_get_size( self->rawframes ); ++ii ) {
             rerr = cpl_frame_set_group( cpl_frameset_get_frame( self->rawframes, ii ), CPL_FRAME_GROUP_RAW );
             pl = cpl_propertylist_load(cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )),0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
             }
             else {
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_DARK_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->dark_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_DARK_CALIB );
    if ( ! self->dark_frame )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract dark_frame frames."
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_DARK_CALIB);
        rerr = SPH_IRD_SCIENCE_DBI_FRAMES_MISSING;
        return rerr;
    }

    else {

         rerr = cpl_frame_set_group( self->dark_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_FLAT_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->flat_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_FLAT_CALIB );
    if ( ! self->flat_frame )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract flat_frame frames."
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_FLAT_CALIB);
        rerr = SPH_IRD_SCIENCE_DBI_FRAMES_MISSING;
        return rerr;
    }

    else {

         rerr = cpl_frame_set_group( self->flat_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_DISTORTION_MAP_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->distmap_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_DISTORTION_MAP_CALIB );
    if ( ! self->distmap_frame )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract distmap_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_DISTORTION_MAP_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->distmap_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_FILTER_TABLE_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->filter_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_FILTER_TABLE_CALIB );
    if ( ! self->filter_frame )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract filter_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_FILTER_TABLE_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->filter_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_STAR_CENTER_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->fcframe = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_STAR_CENTER_CALIB );
    if ( ! self->fcframe )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract fcframe frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_STAR_CENTER_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->fcframe, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_FCTABLE_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->fcasciis = sph_utils_extract_frames( self->inframes, SPH_IRD_TAG_FCTABLE_CALIB );
    if ( ! self->fcasciis )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract fcasciis frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_FCTABLE_CALIB);
        cpl_error_reset();
    }

    else {

         for ( ii = 0; ii < cpl_frameset_get_size( self->fcasciis ); ++ii ) {
             rerr = cpl_frame_set_group( cpl_frameset_get_frame( self->fcasciis, ii ), CPL_FRAME_GROUP_CALIB );
         }
    }

    aframe = cpl_frameset_find( self->inframes, SPH_IRD_TAG_STATIC_BADPIXELMAP_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->static_badpixel_frame = sph_utils_extract_frame( self->inframes, SPH_IRD_TAG_STATIC_BADPIXELMAP_CALIB );
    if ( ! self->static_badpixel_frame )
    {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract static_badpixel_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_IRD_TAG_STATIC_BADPIXELMAP_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->static_badpixel_frame, CPL_FRAME_GROUP_CALIB );
    }

    /* End of code to check frames BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    self->current_raw_frameset = sph_utils_extract_frames_group( self->inframes, CPL_FRAME_GROUP_RAW );
    haskey = 0; /* This is just to remove compiler warnings */
    okval = 0;/* This is just to remove compiler warnings */
    ii = 0; pl = NULL; /* This is just to remove compiler warnings */

    return rerr;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe parameters, and assign them to the sph_ird_science_dbi
           structure.
 @param    self     the sph_ird_science_dbi recipe structure
 @return   error code of the check.

 This checks the parameters that were given in the sph_ird_science_dbi_new function
 as parameters for the correctness and completeness. If the parameters needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_ird_science_dbi_check_params( sph_ird_science_dbi* self ) {
    sph_error_code           	   rerr               = CPL_ERROR_NONE;
    const cpl_parameter*           param              = NULL;

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
    	self->framecomb_parameterlist = NULL;
    }
    self->framecomb_parameterlist = cpl_parameterlist_new();
    /* Code to check parameters GENERATED DO NOT EDIT */

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.outfilename");
    self->science_dbi_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.outfilename_left");
    self->science_dbi_outfilename_left = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.outfilename_left"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.make_template");
    self->make_template = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.make_template"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.keep_fctable");
    self->keep_fctable = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.keep_fctable"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.outfilename_right");
    self->science_dbi_outfilename_right = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.outfilename_right"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.coll_alg");
    self->coll_alg = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.coll_alg"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.use_adi");
    self->flag_adi = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.use_adi"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.use_sdi");
    self->flag_sdi = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.use_sdi"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.minr");
    self->minr = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.minr"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.maxr");
    self->maxr = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.maxr"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.full_frameset_speck");
    self->fullset = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.full_frameset_speck"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.transform_method");
    self->transform_method = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.transform_method"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.filter_method");
    self->filter_method = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.filter_method"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.filter_rad");
    self->filter_radius = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.filter_rad"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.fermi_temp");
    self->fermi_temp = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.fermi_temp"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.butter_pass");
    self->butter_pass = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.butter_pass"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.butter_stop");
    self->butter_stop = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.butter_stop"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "ird.science_dbi.window_size");
    self->window_size = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: ird.science_dbi.window_size"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    /* End of code to check parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return rerr;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Create a new sph_ird_science_dbi structure.

 @param    frameset     the input recipe frames
 @param    parlist      the input recipe parameters

 @return   pointer to newly created structure or NULL in case of error

 This creates a new sph_ird_science_dbi_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_ird_science_dbi* sph_ird_science_dbi_new( cpl_frameset * frameset,
                                              cpl_parameterlist * parlist )
{
    sph_ird_science_dbi*    result      = NULL;
    int                     rerr        = CPL_ERROR_NONE;

    sph_init_erex();
    if ( frameset == NULL || parlist == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return NULL;
    }

    result = cpl_calloc( 1, sizeof(sph_ird_science_dbi) );

    if ( result == NULL ) {
        sph_error_raise( SPH_IRD_SCIENCE_DBI_GENERAL, __FILE__, __func__,
                         __LINE__, SPH_ERROR_ERROR,
                         "Could not allocate the structure.");
        return result;
    }

    result->inframes = frameset;
    result->inparams = parlist;

    rerr = sph_ird_science_dbi_check_frames( result );
    if ( rerr == CPL_ERROR_NONE) rerr = sph_ird_science_dbi_check_params( result );

    if ( rerr != CPL_ERROR_NONE ) {
    	if ( result ) {
    		sph_ird_science_dbi_delete( result );
    	}
    	result = NULL;
    }

    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Delete the sph_ird_science_dbi structure.

 @param    self         the structure tp delete

 @return   error code of the operation

 This creates a new sph_ird_science_dbi_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_ird_science_dbi_delete( sph_ird_science_dbi* self )
{
    sph_error_code          rerr            = CPL_ERROR_NONE;

    if ( self == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return rerr;
    }

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
	    self->framecomb_parameterlist = NULL;
    }

    /* Code to delete recipe pointers GENERATED DO NOT EDIT */

    if ( self->rawframes != NULL ) {
        cpl_frameset_delete( self->rawframes );
        self->rawframes = NULL;
    }
    if ( self->dark_frame != NULL ) {
        cpl_frame_delete( self->dark_frame );
        self->dark_frame = NULL;
    }
    if ( self->flat_frame != NULL ) {
        cpl_frame_delete( self->flat_frame );
        self->flat_frame = NULL;
    }
    if ( self->distmap_frame != NULL ) {
        cpl_frame_delete( self->distmap_frame );
        self->distmap_frame = NULL;
    }
    if ( self->filter_frame != NULL ) {
        cpl_frame_delete( self->filter_frame );
        self->filter_frame = NULL;
    }
    if ( self->fcframe != NULL ) {
        cpl_frame_delete( self->fcframe );
        self->fcframe = NULL;
    }
    if ( self->fcasciis != NULL ) {
        cpl_frameset_delete( self->fcasciis );
        self->fcasciis = NULL;
    }
    if ( self->static_badpixel_frame != NULL ) {
        cpl_frame_delete( self->static_badpixel_frame );
        self->static_badpixel_frame = NULL;
    }
 
    /* End of code to delete pointers BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    if ( self->current_raw_frameset ) {
    	cpl_frameset_delete(self->current_raw_frameset);
	self->current_raw_frameset = NULL;
    }
    sph_polygon_free_all();
    cpl_free(    self);
    return rerr;

}


/**@}*/
