/***********************************************************************************************/
/******** DANGER !! DANGER !! DANGER !! DO NOT EDIT FILE -- IT WAS GENERATED AUTOMATICALLY !!! */
/***********************************************************************************************/

/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#include "sph_common_keywords.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_zpl_basic_collapse_imaging.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"

/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

sph_error_code SPH_ZPL_BASIC_COLLAPSE_IMAGING_GENERAL              = SPH_ZPL_BASIC_COLLAPSE_IMAGING_ERR_START + 0;
sph_error_code SPH_ZPL_BASIC_COLLAPSE_IMAGING_NO_VALID_DIT_LIST    = SPH_ZPL_BASIC_COLLAPSE_IMAGING_ERR_START + 1;

sph_error_code SPH_ZPL_BASIC_COLLAPSE_IMAGING_PARAMETER_MISSING    = SPH_ZPL_BASIC_COLLAPSE_IMAGING_ERR_START + 3;
sph_error_code SPH_ZPL_BASIC_COLLAPSE_IMAGING_FRAMES_MISSING       = SPH_ZPL_BASIC_COLLAPSE_IMAGING_ERR_START + 4;

/*-----------------------------------------------------------------------------
 Private function prototypes
 -----------------------------------------------------------------------------*/

static int sph_zpl_basic_collapse_imaging_create_plugin ( cpl_plugin * );
static int sph_zpl_basic_collapse_imaging_exec_plugin ( cpl_plugin * );
static int sph_zpl_basic_collapse_imaging_destroy_plugin ( cpl_plugin * );

/*-----------------------------------------------------------------------------
 Static variables
 -----------------------------------------------------------------------------*/

static const char* sph_zpl_basic_collapse_imaging_help = 
    "RECIPE INPUT FRAMES: \n"
    "-------------------- \n"
    "Tag                           Min,Max   Description\n"
    "ZPL_BASIC_COLLAPSE_IMAGING_PREPROC_RAW  1, -1   The pre-processed raw data frames.\n"
    "                                        Format: ZPL EXP IMG\n"
    "ZPL_MASTER_BIAS_IMAGING         0,  1   The master bias frame. Should have readout mode as the pre-processed raw data frames.\n"
    "                                        Format: DOUBLE IMAGE\n"
    "ZPL_MASTER_DARK_IMAGING         0,  1   The master dark frame. Should have readout mode as the pre-processed raw data frames.\n"
    "                                        Format: DOUBLE IMAGE\n"
    "ZPL_INT_FLAT_FIELD_IMAGING      0,  1   The intensity flat field frame. Should have readout mode as the pre-processed raw data frames.\n"
    "                                        Format: DOUBLE IMAGE\n"
    "\n"
    "RAW FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Type   Optional  Description\n"
    "ESO DRS PC PROD TYPE          string No        This keyword is mandatory if the pre-processed data are used. \n"
    "                                               As the format of the zimpol pre-processed data is complicated, this keyword was introduced in order \n"
    "                                               to garantee that the pre-processed input frames are polarimetric pre-processed data, \n"
    "                                               produced by the sph_zpl_preproc_imaging utility recipe. \n"
    "\n"
    "DESCRIPTION:\n"
    "------------\n"
    "The recipe produces calibrated and collpased frame in the imaging mode \n"
    "(simple science recipe -- no de-dithering and de-rotation).\n"
    "This recips is an utility recipe and should be only used by off-line data reduction!  \n"
    " \n"
    "The input raw frames used by this recipe are already pre-processed imaging data. \n"
    "These frames  should carry the ZPL_BASIC_IMAGING_PREPROC_RAW tag (normally, it must be raw frames with one \n"
    "particular DIT and filter). The input calibration frames (if any) are the master bias frame with BIAS_IMAGING_CALIB tag, \n"
    "master dark frame with the DARK_IMAGING_CALIB tag, the intensity flat field frame with the IFF_IMAGING CALIB tag.\n"
    "The pre-processed raw frames are transformed first into the DOUBLE IMAGE (8 extensions) format specified as follows:\n"
    "- odd sub-frame image (intensity), badpixel-map, ncomb-map and rms-map;\n"
    "- even  sub-frame image (dark current/non informative), badpixel-map, ncomb-map and rms-map.\n"
    "The master bias, dark and intensity flat field are applied to the double images.\n"
    "\n"
    "The output basic imaging calibrated frames are saved as double image frame.\n"
    "\n"
    "RECIPE PRODUCTS: \n"
    "-------------------- \n"
    "Tag                      Format         Description\n"
    "ZPL_BASIC_COLLAPSE_IMAGING_REDUCEDFITS[Im(8)]    The resulting  basic calibration frame is of the DOUBLE IMAGE format.\n"
    "                                        This frame contains 8 image extensions:\n"
    "                                        - basic imaging calibrated intensity image, badpixel-map, rms-map and  weight-map.\n"
    "                                        - basic imaging calibrated dark image (non-informative), badpixel-map, rms-map and  weight-map.\n"
    "\n"
    "PRODUCT FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Frame          Type    Description\n"
    "\n"
;

/*-----------------------------------------------------------------------------
 Function code
 -----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_zpl_basic_collapse_imaging Create Master Dark Recipe
 *
 * This module provides the recipe plugin and relating functions for the
 * creation of the master dark .
 *
 * @par Synopsis:
 * @code
 *   #include "sph_zpl_basic_collapse_imaging.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*****************************************************************************
 *
 *
 *      First come the static and global functions necessary for the
 *      normal working of the recipe as plugin....
 *
 *
 * ***************************************************************************/
/*----------------------------------------------------------------------------*/
/**
 @brief    Build the list of available plugins, for this module.
 @param    list    the plugin list
 @return   0 if everything is ok, 1 otherwise
 @note     Only this function is exported

 Create the recipe instance and make it available to the application
 using the interface.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe * recipe = cpl_calloc( 1, sizeof *recipe);
    cpl_plugin * plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                        CPL_PLUGIN_API,
                        SPH_ZPL_BASIC_COLLAPSE_IMAGING_VERSION,
                        CPL_PLUGIN_TYPE_RECIPE,
                        "sph_zpl_basic_collapse_imaging",
                        "Calibrate and collapse frames, imaging mode (utility recipe).",
                        sph_zpl_basic_collapse_imaging_help, "Alexey Pavlov",
                        "pavlov@mpia-hd.mpg.de", cpl_get_license("SPHERE DRH","2012"),
                        sph_zpl_basic_collapse_imaging_create_plugin,
                        sph_zpl_basic_collapse_imaging_exec_plugin,
                        sph_zpl_basic_collapse_imaging_destroy_plugin) )
    {
        cpl_msg_error( cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }

    if (cpl_pluginlist_append(list, plugin) )
    {
        cpl_msg_error( cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Setup the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_basic_collapse_imaging_create_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe             = NULL;

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: An error is already set: %s",
                       cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error(cpl_func, "Null plugin");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    if ( plugin != NULL ) {
        recipe = (cpl_recipe *)plugin;
        recipe->parameters = sph_zpl_basic_collapse_imaging_create_paramlist();
        if ( recipe->parameters == NULL ) {
            return SPH_ZPL_BASIC_COLLAPSE_IMAGING_GENERAL;
        }
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Testing for setup of the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Wrapper to make the static plugin creation function accessible from outside
 for testing purposes.
 */
/*----------------------------------------------------------------------------*/
int sph_zpl_basic_collapse_imaging_create_test(cpl_plugin * plugin)
{
    return (int)sph_zpl_basic_collapse_imaging_create_plugin(plugin);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Execute the plugin instance given by the interface
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_basic_collapse_imaging_exec_plugin(cpl_plugin * plugin)
{

    cpl_recipe*             recipe                  = NULL;
    int                     recipe_status           = 0;
    sph_zpl_basic_collapse_imaging*    sph_recipe      = NULL;
    cpl_errorstate          initial_errorstate      = cpl_errorstate_get();

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: A cpl error is already set: %s",
        cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
        return (int)cpl_error_get_code();
    }

    recipe = (cpl_recipe *)plugin;

    if (recipe->parameters == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (recipe->frames == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL frame set");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    sph_recipe = sph_zpl_basic_collapse_imaging_new( recipe->frames, recipe->parameters );
    if ( sph_recipe == NULL ) {
        cpl_msg_error( cpl_func, "SPHERE recipe initialisation call failed. "
                "Please check the SPHERE error messages.");
        sph_end_erex(); //close the sphere system
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    recipe_status = sph_zpl_basic_collapse_imaging_run( sph_recipe );

    sph_zpl_basic_collapse_imaging_delete( sph_recipe );

    if ( !cpl_errorstate_is_equal(initial_errorstate) )
    {
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }
    sph_end_erex(); //close the sphere system
    return recipe_status;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Destroy what has been created by the 'create' function
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_basic_collapse_imaging_destroy_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe;

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    recipe = (cpl_recipe *)plugin;

    cpl_parameterlist_delete(recipe->parameters);

    return 0;
}

/*****************************************************************************
 *
 *
 *      Here come the "Memnber" Functions of the sph_zpl_basic_collapse_imaging struct
 *
 *
 * ***************************************************************************/

/*----------------------------------------------------------------------------*/
/**
 @brief    Create and return the standard parameterlist for the recipe

 @return   the standard parameterlis or NULL if an error occcured.

 This function creates the standard parameterlist for the recipe
 and returns it.

 The return value is NULL when an error occured.
 */
/*----------------------------------------------------------------------------*/
cpl_parameterlist*  sph_zpl_basic_collapse_imaging_create_paramlist(void)
{
    cpl_parameterlist*          result              = NULL;
    cpl_parameter*              p                   = NULL;
    result = cpl_parameterlist_new();

    if (result == NULL)
    {
        cpl_msg_error( cpl_func, "Null parameterlist");
        return NULL;
    }
    else {
        /* Code to set up parameters GENERATED DO NOT EDIT */

        p = cpl_parameter_new_value("zpl.basic_collapse_imaging.outfilename",
                                   CPL_TYPE_STRING,
                                   "The prefix for the output filename for the product. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.basic_collapse_imaging",
                                   "zpl_basic_collapse_imaging.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.basic_collapse_imaging.subtract_overscan",
                                   CPL_TYPE_BOOL,
                                   "Flag to set if the overscan mean values must be subtracted from pre-processed data (TRUE) "
                                   "Note that this parameter is applied if pre-processed data containt overscan table "
                                   ,"zpl.basic_collapse_imaging",
                                   1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("zpl.basic_collapse_imaging.coll_alg",
                                   CPL_TYPE_INT,
                                   "Set the collapse algorithm. The available algorithms: 0 = Mean, 1 = Median, 2 = Clean Mean. Default is 2 = Clean Mean "
                                   ,"zpl.basic_collapse_imaging",
                                   2 ,
                                   3,0,1,2);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.basic_collapse_imaging.coll_alg.clean_mean.reject_high",
                                   CPL_TYPE_INT,
                                   "The number of pixels to reject when combining frames at the high end. Number of input frames must be > reject_high +reject_low "
                                   ,"zpl.basic_collapse_imaging",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.basic_collapse_imaging.coll_alg.clean_mean.reject_low",
                                   CPL_TYPE_INT,
                                   "The number of pixels to reject when combining frames at the low end. Number of input frames must be > reject_high +reject_low "
                                   ,"zpl.basic_collapse_imaging",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.basic_collapse_imaging.clean_mean.sigma",
                                   CPL_TYPE_DOUBLE,
                                   "The number of pixels to reject when combining frames in sigma from median. NOT SUPPORTED YET! "
                                   ,"zpl.basic_collapse_imaging",
                                   5.0 ,
                                   0.0,200.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.basic_collapse_imaging.sigma_clip",
                                   CPL_TYPE_DOUBLE,
                                   "The sigma clipping value for static badpixel detection. Default is 5. "
                                   ,"zpl.basic_collapse_imaging",
                                   5.0 ,
                                   0.0,200.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.basic_collapse_imaging.cfg_filename",
                                   CPL_TYPE_STRING,
                                   "The configuration filename containing information about the planes which should be ignored at collapsing. "
                                   ,"zpl.basic_collapse_imaging",
                                   "zpl_basic_collapse_imaging.cfg" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        /* End of code to set up parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    }


    return result;


}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe frames, and assign them to the sph_zpl_basic_collapse_imaging
           structure.
 @param    self     the sph_zpl_basic_collapse_imaging recipe structure
 @return   error code of the check.

 This checks the frames that were given in the sph_zpl_basic_collapse_imaging_new function
 as parameters for the correctness and completeness. If the frames needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code  sph_zpl_basic_collapse_imaging_check_frames( sph_zpl_basic_collapse_imaging* self )
{
    sph_error_code           rerr            = CPL_ERROR_NONE;
    int                      ii	             = 0;
    cpl_frame*               aframe          = NULL;
    cpl_propertylist*        pl              = NULL;
    int	                     haskey          = 0;
    int                      okval           = 0;
    /* Code to check frames GENERATED DO NOT EDIT */

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->rawframes = sph_utils_extract_frames( self->inframes, SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW );
    if ( ! self->rawframes )
    {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_ERROR,
                         "Could not extract rawframes frames."
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_BASIC_COLLAPSE_IMAGING_PREPROC_RAW);
        rerr = SPH_ZPL_BASIC_COLLAPSE_IMAGING_FRAMES_MISSING;
        return rerr;
    }

    else {

         for ( ii = 0; ii < cpl_frameset_get_size( self->rawframes ); ++ii ) {
             rerr = cpl_frame_set_group( cpl_frameset_get_frame( self->rawframes, ii ), CPL_FRAME_GROUP_RAW );
             pl = cpl_propertylist_load(cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )),0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
             }
             else {
                 haskey = 0;okval = 0;
                 if ( cpl_propertylist_has(pl,"ESO DRS PC PROD TYPE") )  {
                     haskey = 1;
                     if ( cpl_propertylist_get_type(pl,"ESO DRS PC PROD TYPE") == CPL_TYPE_STRING )  {
                         okval = 1; //Type is string so always ok
                     } 
                     if ( haskey == 0 || okval == 0 ) {
                         sph_error_raise(CPL_ERROR_ILLEGAL_INPUT,
                            __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                         "Could not read a keyword ESO DRS PC PROD TYPE from file %s. "
                         "that has __type string "
                         "and values between 0.0 and 0.0 "
                         "The recipe requires this. ",
                         cpl_frame_get_filename(cpl_frameset_get_frame( self->rawframes, ii )));
                     }
                 }
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_bias_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    if ( ! self->master_bias_frame )
    {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract master_bias_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_BIAS_IMAGING_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_bias_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    if ( ! self->master_dark_frame )
    {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract master_dark_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_DARK_IMAGING_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->intensity_flat_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_CALIB );
    if ( ! self->intensity_flat_frame )
    {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_WARNING,
                         "Could not extract intensity_flat_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this is just a warning. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFF_IMAGING_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    }

    /* End of code to check frames BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    self->current_raw_frameset = sph_utils_extract_frames_group( self->inframes, CPL_FRAME_GROUP_RAW );
    haskey = 0; /* This is just to remove compiler warnings */
    okval = 0;/* This is just to remove compiler warnings */
    ii = 0; pl = NULL; /* This is just to remove compiler warnings */

    return rerr;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe parameters, and assign them to the sph_zpl_basic_collapse_imaging
           structure.
 @param    self     the sph_zpl_basic_collapse_imaging recipe structure
 @return   error code of the check.

 This checks the parameters that were given in the sph_zpl_basic_collapse_imaging_new function
 as parameters for the correctness and completeness. If the parameters needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_zpl_basic_collapse_imaging_check_params( sph_zpl_basic_collapse_imaging* self ) {
    sph_error_code           	   rerr               = CPL_ERROR_NONE;
    const cpl_parameter*           param              = NULL;

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
    	self->framecomb_parameterlist = NULL;
    }
    self->framecomb_parameterlist = cpl_parameterlist_new();
    /* Code to check parameters GENERATED DO NOT EDIT */

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.outfilename");
    self->basic_collapse_imaging_outfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.basic_collapse_imaging.outfilename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.subtract_overscan");
    self->subtract_overscan = cpl_parameter_get_bool( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.basic_collapse_imaging.subtract_overscan"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.coll_alg");
    self->coll_alg = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.basic_collapse_imaging.coll_alg"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.coll_alg.clean_mean.reject_high");
    self->clean_mean_reject_high = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_high",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.coll_alg.clean_mean.reject_low");
    self->clean_mean_reject_low = cpl_parameter_get_int( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_low",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.clean_mean.sigma");
    self->clean_mean_sigma = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.sigma",
                                                       CPL_TYPE_DOUBLE,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_double( param ) ,
                                                       0.0,200.0
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.sigma_clip");
    self->sigma_clip = cpl_parameter_get_double( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.basic_collapse_imaging.sigma_clip"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.basic_collapse_imaging.cfg_filename");
    self->basic_collapse_imaging_cfgfilename = cpl_parameter_get_string( ( const cpl_parameter*  )param);
    if ( cpl_error_get_code() != CPL_ERROR_NONE ) {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_PARAMETER_MISSING,
                          __FILE__, __func__,
                          __LINE__, SPH_ERROR_ERROR,
                          "Could not retrieve the input parameters."
                          "Make sure that: zpl.basic_collapse_imaging.cfg_filename"
                          "is included in parameter list passed"
                          "to this recipe.");
        rerr = cpl_error_get_code();
        return rerr;
    }

    /* End of code to check parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return rerr;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Create a new sph_zpl_basic_collapse_imaging structure.

 @param    frameset     the input recipe frames
 @param    parlist      the input recipe parameters

 @return   pointer to newly created structure or NULL in case of error

 This creates a new sph_zpl_basic_collapse_imaging_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_zpl_basic_collapse_imaging* sph_zpl_basic_collapse_imaging_new( cpl_frameset * frameset,
                                              cpl_parameterlist * parlist )
{
    sph_zpl_basic_collapse_imaging*    result      = NULL;
    int                     rerr        = CPL_ERROR_NONE;

    sph_init_erex();
    if ( frameset == NULL || parlist == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return NULL;
    }

    result = cpl_calloc( 1, sizeof(sph_zpl_basic_collapse_imaging) );

    if ( result == NULL ) {
        sph_error_raise( SPH_ZPL_BASIC_COLLAPSE_IMAGING_GENERAL, __FILE__, __func__,
                         __LINE__, SPH_ERROR_ERROR,
                         "Could not allocate the structure.");
        return result;
    }

    result->inframes = frameset;
    result->inparams = parlist;

    rerr = sph_zpl_basic_collapse_imaging_check_frames( result );
    if ( rerr == CPL_ERROR_NONE) rerr = sph_zpl_basic_collapse_imaging_check_params( result );

    if ( rerr != CPL_ERROR_NONE ) {
    	if ( result ) {
    		sph_zpl_basic_collapse_imaging_delete( result );
    	}
    	result = NULL;
    }

    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Delete the sph_zpl_basic_collapse_imaging structure.

 @param    self         the structure tp delete

 @return   error code of the operation

 This creates a new sph_zpl_basic_collapse_imaging_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_zpl_basic_collapse_imaging_delete( sph_zpl_basic_collapse_imaging* self )
{
    sph_error_code          rerr            = CPL_ERROR_NONE;

    if ( self == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return rerr;
    }

    if ( self->framecomb_parameterlist != NULL ) {
    	cpl_parameterlist_delete( self->framecomb_parameterlist );
	    self->framecomb_parameterlist = NULL;
    }

    /* Code to delete recipe pointers GENERATED DO NOT EDIT */

    if ( self->rawframes != NULL ) {
        cpl_frameset_delete( self->rawframes );
        self->rawframes = NULL;
    }
    if ( self->master_bias_frame != NULL ) {
        cpl_frame_delete( self->master_bias_frame );
        self->master_bias_frame = NULL;
    }
    if ( self->master_dark_frame != NULL ) {
        cpl_frame_delete( self->master_dark_frame );
        self->master_dark_frame = NULL;
    }
    if ( self->intensity_flat_frame != NULL ) {
        cpl_frame_delete( self->intensity_flat_frame );
        self->intensity_flat_frame = NULL;
    }
 
    sph_double_image_delete(self->basic_collpase_imaging);
    /* End of code to delete pointers BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    if ( self->current_raw_frameset ) {
    	cpl_frameset_delete(self->current_raw_frameset);
	self->current_raw_frameset = NULL;
    }
    sph_polygon_free_all();
    cpl_free(    self);
    return rerr;

}


/**@}*/
