/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h>
#include "sph_error.h"
#include "sph_master_frame.h"
#include "sph_polynomial_distortion_model.h"
#include "sph_framecombination.h"
#include "sph_ird_instrument_model.h"
#include "sph_zpl_exposure.h"
#include "sph_fft.h"
#ifndef SPH_COMMON_SCIENCE_H_
#define SPH_COMMON_SCIENCE_H_

sph_error_code
sph_common_science_reject_badpix(
        cpl_imagelist* imlist,
        cpl_imagelist* badpixs);
cpl_imagelist*
sph_common_science_calc_weights(cpl_imagelist* imlist,
        cpl_imagelist* badpixs, cpl_image* medim);
sph_error_code
sph_common_science_reject_badpix_image(
        cpl_image* im,
        cpl_image* badpixs);
cpl_frame*
sph_common_science_get_current_raw_frame(void);
sph_error_code
sph_common_science_set_weights_zero(cpl_imagelist* weights, cpl_imagelist* badpixs);
cpl_image*
sph_common_science_remap_badpix(cpl_image* inbadpix,
        double angle, double centx,
        double centy,double dx, double dy,
        sph_polynomial_distortion_model* distmap,
        double scale,
        sph_ird_instrument_model* irdmodel);

cpl_vector*
sph_common_science_extract_angles( cpl_frameset* fctables );

sph_error_code
sph_common_science_combine_imlists(
        sph_collapse_algorithm coll_alg,
        cpl_imagelist* imlist,
        cpl_imagelist* bpixlist,
        cpl_imagelist* wlist,
        sph_master_frame** result);

sph_master_frame*
sph_common_science_calibrate_raw( cpl_image* rawimage,
           sph_master_frame* dark,
           sph_master_frame* flat);

sph_master_frame*
sph_common_science_combine(
        cpl_frameset* inframeset,
        sph_collapse_algorithm coll_alg,
        unsigned int ext_im,
        int ext_bp,
        int ext_wm);
int
sph_common_science_get_current_plane(void);

sph_master_frame*
sph_common_science_combine_single_cube(
        cpl_frame* cube_frame,
        sph_collapse_algorithm coll_alg,
        unsigned int ext_im,
        int ext_bp,
        int ext_wm);

sph_double_image*
sph_common_science_combine_weight_double_image_frameset( cpl_frameset* inframes);
cpl_image*
sph_common_science_get_next_raw(cpl_frameset* rawframes, int ext);
sph_master_frame*
sph_common_science_get_next_master_frame(cpl_frameset* frames);
sph_zpl_exposure*
sph_common_science_get_next_zpl_exposure(cpl_frameset* rawframes);
sph_zpl_exposure_imaging*
sph_common_science_get_next_zpl_exposure_imaging(cpl_frameset* rawframes);
sph_double_image*
sph_common_science_get_next_double_image(cpl_frameset* frames);
int
sph_common_science_get_nraws(cpl_frameset* rawframes, int ext);

#endif /* SPH_COMMON_SCIENCE_H_ */
