/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifndef SPH_DOUBLE_IMAGE_H_
#define SPH_DOUBLE_IMAGE_H_
#include "sph_master_frame.h"
#include "sph_zpl_exposure_imaging.h"
#include "sph_error.h"

typedef struct _sph_double_image_
{
	sph_master_frame* 		iframe;
	sph_master_frame*		pframe;
	cpl_propertylist*		properties;
	cpl_propertylist*		qclist;
} sph_double_image;

/*-----------------------------------------------------------------------------
 Function prototypes
 -----------------------------------------------------------------------------*/

sph_double_image* sph_double_image_new( int nx, int ny );
sph_double_image* sph_double_image_new_empty( void );
sph_double_image* sph_double_image_new_from_master_frames( sph_master_frame* mf_odd, sph_master_frame* mf_even);
sph_double_image* sph_double_image_new_zpl_subframes( sph_master_frame* framein, short phase_mode );
sph_double_image* sph_double_image_new_from_zpl_exposure_imaging( sph_zpl_exposure_imaging* zplexpimg );

sph_double_image* sph_double_image_load( const char* czFilename, int plane );
sph_double_image* sph_double_image_duplicate ( const sph_double_image* self);
sph_error_code	  sph_double_image_add_double_image( sph_double_image* self, sph_double_image* diframe);
sph_error_code	  sph_double_image_subtract_double_image( sph_double_image* self, sph_double_image* diframe);
sph_error_code	  sph_double_image_divide_double_image( sph_double_image* self, sph_double_image* dimage);
sph_error_code 	  sph_double_image_divide_double( sph_double_image* self, double value);
sph_error_code	  sph_double_image_combine_double_image (sph_double_image* self, sph_double_image* diframe_minus);
cpl_error_code 	  sph_double_image_quality_check( sph_double_image* self );
cpl_error_code 	  sph_double_image_mask_sigma( sph_double_image* self, double sigma);

sph_master_frame* sph_double_image_get_odd_master_frame ( sph_double_image* self );
sph_master_frame* sph_double_image_get_even_master_frame ( sph_double_image* self );
sph_master_frame* sph_double_image_get_iframe_master_frame (sph_double_image* self);
sph_master_frame* sph_double_image_get_pframe_master_frame (sph_double_image* self);
sph_master_frame* sph_double_image_get_iframe_master_frame_duplicate (sph_double_image* self);
sph_master_frame* sph_double_image_get_pframe_master_frame_duplicate (sph_double_image* self);
sph_master_frame*
sph_double_image_calculate_pframe_slash_iframe_master_frame( sph_double_image* self);

cpl_frameset*     sph_double_image_create_double_image_frameset_from_zplexp_imaging_cubes( cpl_frameset* inframes );
cpl_frameset*     sph_double_image_create_double_image_frameset_from_zplexp_imaging_frameset( cpl_frameset* inframes );
sph_error_code 	  sph_double_image_expand2y( sph_double_image* self );
cpl_boolean 	  sph_double_image_is_squared( sph_double_image* self );
sph_error_code    sph_double_image_add_topbottom_lines( sph_double_image* self );
sph_error_code	  sph_double_image_cutedge_columns( sph_double_image* self );



sph_error_code
sph_double_image_save_dfs(
        sph_double_image* self,
        const char* outfilename,
        cpl_frameset* allframes,
        cpl_frame* template_frame,
        cpl_parameterlist* params,
        const char* tag,
        const char* recipe,
        const char* pipename,
        cpl_propertylist* pli);

sph_error_code sph_double_image_save( sph_double_image* self,
		const char* czFilename, cpl_propertylist* pli, unsigned mode );
sph_error_code sph_double_image_delete( sph_double_image* self );

#endif /* SPH_DOUBLE_IMAGE_H_ */
