/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <cpl.h>
#include "sph_error.h"
#include "sph_point.h"
#ifndef SPH_POINT_PATTERN_H_
#define SPH_POINT_PATTERN_H_
typedef struct _sph_point_pattern_ {
	cpl_vector* points_x;
	cpl_vector* points_y;
} sph_point_pattern;

sph_point_pattern*
sph_point_pattern_new(void);


sph_point_pattern*
sph_point_pattern_new_from_table(
        cpl_frame* framein,
        int ext );

sph_error_code
sph_point_pattern_add_point(
        sph_point_pattern* self,
        double x,
        double y );

sph_point_pattern*
sph_point_pattern_new_from_image(
        cpl_image* image,
        cpl_image* errim,
        double sigma,
        double point_width_estimate);

sph_error_code
sph_point_pattern_rotate(
        sph_point_pattern* self,
        double cx, double cy,
        double angle );
sph_error_code
sph_point_pattern_scale_around(
        sph_point_pattern* self,
        double cx, double cy,
        double scale );
sph_error_code
sph_point_pattern_clear( sph_point_pattern* self );
sph_error_code
sph_point_pattern_centralise( sph_point_pattern* self );
sph_point*
sph_point_pattern_get_point( sph_point_pattern* self, int n );
sph_point*
sph_point_pattern_find_closest( sph_point_pattern* self, double x, double y );
cpl_image*
sph_point_pattern_create_image( sph_point_pattern* self, int nx, int ny, double pointsize );
int
sph_point_pattern_get_size( sph_point_pattern* self );
int
sph_point_pattern_find_closest_neighbour( sph_point_pattern* self, int n, double* pdist );
sph_error_code
sph_point_pattern_save( sph_point_pattern* self, const char* filename );
void sph_point_pattern_delete( sph_point_pattern* self );
int
sph_point_pattern_get_central_point( sph_point_pattern* self, double* x, double* y );
sph_error_code
sph_point_pattern_add_offset( sph_point_pattern* self, double x0, double y0 );
sph_error_code
sph_point_pattern_centralise_wrt_central_point( sph_point_pattern* self );
sph_error_code
sph_point_pattern_save_dfs(
        sph_point_pattern* self,
        const char* outfilename,
        cpl_frameset* allframes,
        cpl_frame* template_frame,
        cpl_parameterlist* params,
        const char* tag,
        const char* recipe,
        const char* pipename,
        cpl_propertylist* plist );

sph_point_pattern*
sph_point_pattern_load_ascii(const char* filename);

sph_error_code
sph_point_pattern_save_ascii(
        sph_point_pattern* self,
        const char* filename );

#ifdef SPH_POINT_PATTERN_C_
static
int
__sph_point_pattern_add_gauss_to_image( cpl_image* inimage,
        double dxpos,
        double dypos,
        double psize,
        double value );
#endif
#endif /* SPH_POINT_PATTERN_H_ */
