/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */


#ifndef SPH_POLYGON_H_
#define SPH_POLYGON_H_
#include "sph_error.h"
#include "sph_point.h"
#include "sph_line.h"


typedef struct _sph_polygon_ {
    unsigned short   npoints;
    sph_point**      points;
} sph_polygon;

typedef struct _sph_polygon_workspace_ {
    sph_point*      point1;
    sph_point*      point2;
    sph_point*      point3;
    sph_line*       line1;
    sph_line*       line2;
    sph_line*       line3;
    sph_polygon*    poly1;
} sph_polygon_workspace;


sph_point*
sph_polygon_get_point(
        sph_polygon* self,
        int p );

sph_polygon*
sph_polygon_new( void );

sph_error_code
sph_polygon_sort( sph_polygon* self );


void
sph_polygon_free_static( void );

sph_polygon*
sph_polygon_duplicate( sph_polygon* poly );


sph_error_code
sph_polygon_add_point(
        sph_polygon* self,
        double x, double y );

sph_error_code
sph_polygon_clip_to_target(
        sph_polygon* self,
        sph_polygon* target,
        sph_polygon_workspace* ws);

double
sph_polygon_calculate_overlap(
        sph_polygon* self,
        sph_polygon* target,
        sph_polygon_workspace* ws );

short sph_polygon_test_inside( sph_polygon* self, double x, double y );

double
sph_polygon_get_top(sph_polygon* self);

double
sph_polygon_get_bottom(sph_polygon* self);

double
sph_polygon_get_left(sph_polygon* self);

double
sph_polygon_get_right(sph_polygon* self);

double
sph_polygon_get_midx( sph_polygon* self );

double
sph_polygon_get_midy( sph_polygon* self );

short
sph_polygon_test_same_side(
        sph_polygon* self,
        int ee,
        sph_point* p,
        sph_line* work_line
        );

sph_error_code
sph_polygon_rotate_around( sph_polygon* self,
                                          double x, double y,
                                          double cosangle,
                                          double sinangle,
                                          double dx, double dy );

sph_line*
sph_polygon_get_edge( sph_polygon* self, int edge );

sph_error_code
sph_polygon_shift( sph_polygon* self,
                   double dx, double dy );

sph_error_code
sph_polygon_enlarge( sph_polygon* self, double fac );

double
sph_polygon_area( sph_polygon* self);

void
sph_polygon_delete( sph_polygon* self );

void
sph_polygon_workspace_delete( sph_polygon_workspace* self );

sph_polygon_workspace*
sph_polygon_workspace_new( void );

void
sph_polygon_free_all( void );

#ifdef _SPH_POLYGON_C_
static
void
__sph_poly_add_point_static( sph_polygon* newpol, sph_point* p, int pointc );

static
double __sph_line_intersect_point_y( sph_line* lineA, sph_line* lineB );
static
double __sph_line_intersect_point_x( sph_line* lineA, sph_line* lineB );
static
int __sph_polygon_get_edge_inplace( sph_polygon* self, int edge,
        sph_line* inplace );
#endif

#endif /*SPH_POLYGON_H_*/
