/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_ifs_instrument_model.h"
#include "sph_ldt.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"
#include "sph_overlap_calculator.h"
#include "cutest_test_ifs_instrument_model.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup Testing the IFS instrument model simple simulation code
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_test_ifs_instrument_model",
                          cutest_init_test_ifs_instrument_model_testsuite,
                          cutest_clean_test_ifs_instrument_model_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_test_ifs_instument_model_delete",
                             cutest_test_ifs_instrument_model_delete ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_test_ifs_instument_model_disperse",
                             cutest_test_ifs_instrument_model_disperse ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
#ifdef SPHERE_MANUAL
    CU_console_run_tests();
#else
    CU_basic_run_tests();
#endif
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}






/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_test_ifs_instrument_model_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_test_ifs_instrument_model_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_ifs_instrument_model_disperse() {
    /*
     * This whole test needs some tidying up.
     */
    sph_test_ifs_instrument_model*        insmodel     = NULL;
    cpl_image*                            inimage        = NULL;
    cpl_image*                            digimage    = NULL;
    sph_master_frame*                    masterf        = NULL;
    int                                    nx = 512;
    int                                    ny = 512;
    double                                total_flux = 0.0;
    cpl_image*                            comp_image = NULL;
    cpl_propertylist*                       plist = NULL;
    sph_polygon*                            poly_pixel = NULL;
    sph_polygon*                            poly_region = NULL;
    sph_polygon_workspace*                  ws      = NULL;
    /* Setup:
     * This sets up an input image as it would be before the lenslet array.
     * The image consists of two gaussians. The total flux is saved
     * for later test verification.
     */
    ws = sph_polygon_workspace_new();
    inimage = sph_test_image_tools_create_flat_image_double(nx,ny,0.0);
    cpl_image_set(inimage,nx/2,ny/2,1.0);
    total_flux = cpl_image_get_absflux(inimage);
    CU_ASSERT_DOUBLE_EQUAL_FATAL(total_flux,1.0, 0.01);
    insmodel = sph_test_ifs_instrument_model_new( nx, 161.5, 1 );
    CU_ASSERT_PTR_NOT_NULL_FATAL( insmodel );

    // Preheck that central pixel is in central spectral region
    poly_region = sph_ifs_lenslet_model_get_spec_region_pixel_coords(
            insmodel->lensmodel,0,0,0.0,0.0);
    poly_pixel = sph_polygon_new();
    sph_polygon_add_point(poly_pixel,nx/2.0,ny/2.0);
    sph_polygon_add_point(poly_pixel,nx/2.0,ny/2.0 + 1.0);
    sph_polygon_add_point(poly_pixel,nx/2.0 + 1.0,ny/2.0 + 1.0);
    sph_polygon_add_point(poly_pixel,nx/2.0 + 1.0,ny/2.0);
    CU_ASSERT_DOUBLE_EQUAL_FATAL(
            sph_polygon_calculate_overlap(poly_pixel,poly_region,ws),
            1.0,0.00001
            );

    sph_polygon_delete(poly_pixel); poly_pixel = NULL;
    /* Execution:
     * The image is dispersed through the lenslet array.
     */
    //printf( "start with dispersing.\n");

    digimage = sph_test_ifs_instrument_model_disperse( insmodel, inimage, 0.0, 0.0 );

    //printf( "Done with dispersing.\n");
    /* Verification
     * It is verified that
     *  -- the answer is not NULL
     *  -- a sensible LDT can be constructed.
     *  -- The flux in this LDT is the same as the input flux
     *     to an accuracy of 0.1 per cent.
     */
    CU_ASSERT_DOUBLE_EQUAL(cpl_image_get_flux(digimage),total_flux,0.001);

    CU_ASSERT_PTR_NOT_NULL_FATAL( digimage );
    cpl_image_save( digimage,
            "dispersed.fits",
            CPL_TYPE_DOUBLE,
            plist = sph_ifs_lenslet_model_get_as_propertylist( insmodel->lensmodel ),
            CPL_IO_CREATE );

    cpl_propertylist_delete(plist); plist = NULL;
    comp_image = cpl_image_load( "data/dispersed_comp.fits", CPL_TYPE_DOUBLE, 0, 0 );
    CU_ASSERT_PTR_NOT_NULL_FATAL(comp_image);

    cpl_image_subtract( comp_image, digimage);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_absflux( comp_image ) , 0.0, 0.1 );
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_mean( comp_image ) , 0.0, 0.1 );

    cpl_image_delete(comp_image); comp_image = NULL;


    sph_test_ifs_instrument_model_delete( insmodel );
    cpl_image_delete(digimage);
    cpl_image_delete(inimage);
    cpl_image_delete(comp_image); comp_image = NULL;
    sph_master_frame_delete(masterf);
    sph_polygon_workspace_delete(ws);
    insmodel = NULL;
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_test_ifs_instrument_model_delete() {
    sph_test_ifs_instrument_model*        insmodel     = NULL;
    sph_ifs_lenslet_model*                model = NULL;

    model = sph_ifs_lenslet_model_new();
    insmodel = sph_test_ifs_instrument_model_new_from_lenslet_model(model);
    CU_ASSERT_PTR_NOT_NULL( insmodel );
    CU_ASSERT_PTR_NOT_NULL( insmodel->lensmodel );

    sph_test_ifs_instrument_model_delete( insmodel );
    insmodel = NULL;

    return;
}

/**@}*/
