/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_irdis_model.h"
#include "sph_utils.h"
#include "sph_common_keywords.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup Testing the IFS instrument model simple simulation code
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
int cutest_init_test_irdis_model_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de", CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
int cutest_clean_test_irdis_model_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
void cutest_test_irdis_model_new() {
    sph_test_irdis_model*        insmodel     = NULL;
    sph_test_ngc_ir_simulator*   ngc            = NULL;
    ngc = sph_test_ngc_ir_simulator_new(512,256);

    insmodel = sph_test_irdis_model_new(ngc);
    CU_ASSERT_PTR_NOT_NULL( insmodel );

    sph_test_irdis_model_delete( insmodel );
    insmodel = NULL;
    sph_test_ngc_ir_simulator_delete(ngc);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
void cutest_test_irdis_model_create_pdt_single_window() {
    sph_test_irdis_model*        insmodel     = NULL;
    sph_pixel_description_table*    pdt = NULL;
    sph_spectral_region*            reg = NULL;
    sph_test_ngc_ir_simulator*   ngc            = NULL;
    ngc = sph_test_ngc_ir_simulator_new(512,256);
    insmodel = sph_test_irdis_model_new(ngc);

    CU_ASSERT_PTR_NOT_NULL( insmodel );
    sph_test_irdis_model_set_mode( insmodel, SPH_TEST_IRDIS_INSMODE_SPEC_LRS );
    pdt = sph_test_irdis_model_create_pdt_single_window(insmodel);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pdt);
    sph_pixel_description_table_save(pdt,"irdis_tab.fits",NULL);
    CU_ASSERT_EQUAL(pdt->nregions,1);
    reg = sph_pixel_description_table_get_region(pdt,1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( reg );
    CU_ASSERT_EQUAL(reg->maxx-reg->minx,insmodel->specwin_maxx-insmodel->specwin_minx);
    CU_ASSERT_EQUAL(reg->maxy-reg->miny,insmodel->specwin_maxy-insmodel->specwin_miny);

    sph_test_irdis_model_delete( insmodel );
    sph_pixel_description_table_delete(pdt); pdt = NULL;
    insmodel = NULL;
    sph_test_ngc_ir_simulator_delete(ngc);

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
void cutest_test_irdis_model_create_pdt() {
    sph_test_irdis_model*        insmodel     = NULL;
    sph_pixel_description_table*    pdt = NULL;
    sph_spectral_region*            reg = NULL;
    sph_test_ngc_ir_simulator*   ngc            = NULL;
    ngc = sph_test_ngc_ir_simulator_new(512,256);

    insmodel = sph_test_irdis_model_new(ngc);

    CU_ASSERT_PTR_NOT_NULL( insmodel );
    sph_test_irdis_model_set_mode( insmodel, SPH_TEST_IRDIS_INSMODE_SPEC_LRS );
    pdt = sph_test_irdis_model_create_pdt(insmodel);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pdt);
    sph_pixel_description_table_save(pdt,"irdis_tab_whole.fits",NULL);
    CU_ASSERT_EQUAL(pdt->nregions,2);
    reg = sph_pixel_description_table_get_region(pdt,1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( reg );
    CU_ASSERT_EQUAL(reg->maxx-reg->minx,insmodel->specwin_maxx-insmodel->specwin_minx);
    CU_ASSERT_EQUAL(reg->maxy-reg->miny,insmodel->specwin_maxy-insmodel->specwin_miny);

    sph_test_irdis_model_delete( insmodel );
    sph_pixel_description_table_delete(pdt); pdt = NULL;
    insmodel = NULL;
    sph_test_ngc_ir_simulator_delete(ngc);

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
void cutest_test_irdis_model_process_image_specmode() {
    sph_test_irdis_model*        insmodel     = NULL;
    cpl_image*                    inimage = NULL;
    cpl_image*                    result = NULL;
    sph_pixel_description_table*    pdt = NULL;
    cpl_frame*                    inframe = NULL;
    cpl_frameset*                inframeset = NULL;
    cpl_frame*                    dumframe = NULL;
    int xx,yy;
    sph_test_ngc_ir_simulator*   ngc            = NULL;
    ngc = sph_test_ngc_ir_simulator_new(512,256);

    insmodel = sph_test_irdis_model_new(ngc);

    CU_ASSERT_PTR_NOT_NULL( insmodel );

    sph_test_irdis_model_set_mode( insmodel, SPH_TEST_IRDIS_INSMODE_SPEC_LRS );
    pdt = sph_test_irdis_model_create_pdt_single_window(insmodel);
    CU_ASSERT_PTR_NOT_NULL_FATAL(pdt);

    inimage = cpl_image_new( pdt->nx, pdt->ny, CPL_TYPE_DOUBLE );
    for (xx = 0; xx < cpl_image_get_size_x(inimage); ++xx) {
        for (yy = 0; yy < cpl_image_get_size_y(inimage); ++yy) {
            cpl_image_set(inimage,xx+1,yy+1,yy + 100);
        }
    }

    result = sph_test_irdis_model_process_image(insmodel,inimage);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);

    dumframe = sph_test_frame_image(result, CPL_TYPE_DOUBLE, "IRDIS TEST", CPL_FRAME_GROUP_RAW );


    sph_test_irdis_model_delete( insmodel );
    sph_pixel_description_table_delete(pdt); pdt = NULL;
    insmodel = NULL;
    cpl_image_delete(result); result = NULL;
    cpl_image_delete( inimage ); inimage = NULL;
    sph_test_ngc_ir_simulator_delete(ngc);
    cpl_frame_delete(dumframe);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
void cutest_test_irdis_model_process_image_dbi() {
    sph_test_irdis_model*        insmodel     = NULL;
    cpl_image*                    inimage = NULL;
    cpl_image*                    result = NULL;
    sph_pixel_description_table*    pdt = NULL;
    cpl_frame*                    inframe = NULL;
    cpl_frameset*                inframeset = NULL;
    double                        flux0    = 0.0;
    double                        flux    = 0.0;
    cpl_frame*                    dumframe = NULL;
    int xx,yy;
    sph_test_ngc_ir_simulator*   ngc            = NULL;
    ngc = sph_test_ngc_ir_simulator_new(512,256);

    insmodel = sph_test_irdis_model_new(ngc);

    CU_ASSERT_PTR_NOT_NULL( insmodel );

    sph_test_irdis_model_set_mode( insmodel, SPH_TEST_IRDIS_INSMODE_DBI );

    inimage = cpl_image_new( insmodel->nx, insmodel->ny, CPL_TYPE_DOUBLE );
    for (xx = 0; xx < cpl_image_get_size_x(inimage); ++xx) {
        for (yy = 0; yy < cpl_image_get_size_y(inimage); ++yy) {
            cpl_image_set(inimage,xx+1,yy+1,100.0);
        }
    }
    flux0 = cpl_image_get_flux(inimage);


    result = sph_test_irdis_model_process_image(insmodel,inimage);
    CU_ASSERT_PTR_NOT_NULL_FATAL(result);

    dumframe = sph_test_frame_image(result, CPL_TYPE_DOUBLE, "IRDIS_DBI_TEST", CPL_FRAME_GROUP_RAW );

    flux0 = flux0 / (insmodel->nx * insmodel->ny);
    flux0 = flux0 * SPH_PI * ( insmodel->radius_left * insmodel->radius_left + insmodel->radius_right * insmodel->radius_right );
    flux = cpl_image_get_flux(result);
    CU_ASSERT_DOUBLE_EQUAL(flux,flux0,SPH_PI * 100 * (2 * insmodel->radius_left + 2 * insmodel->radius_right));
    sph_test_irdis_model_delete( insmodel );
    insmodel = NULL;
    cpl_image_delete( result ); result = NULL;
    cpl_image_delete( inimage ); inimage = NULL;
    sph_test_ngc_ir_simulator_delete(ngc);
    cpl_frame_delete(dumframe);
    return;
}


//*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
void cutest_test_irdis_model_delete() {
    sph_test_irdis_model*        insmodel     = NULL;
    sph_test_ngc_ir_simulator*   ngc            = NULL;
    ngc = sph_test_ngc_ir_simulator_new(512,256);

    insmodel = sph_test_irdis_model_new(ngc);
    CU_ASSERT_PTR_NOT_NULL( insmodel );

    sph_test_irdis_model_delete( insmodel );
    insmodel = NULL;
    sph_test_ngc_ir_simulator_delete(ngc);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("Testsuite for sph_test_irdis_model",
                          cutest_init_test_irdis_model_testsuite,
                          cutest_clean_test_irdis_model_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_test_irdis_model_new",
                             cutest_test_irdis_model_new ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_test_irdis_model_create_pdt",
                             cutest_test_irdis_model_create_pdt ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_test_irdis_model_create_pdt_single_window",
                             cutest_test_irdis_model_create_pdt_single_window ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_test_irdis_model_process_image_specmode",
                             cutest_test_irdis_model_process_image_specmode ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_test_irdis_model_process_image_dbi",
                             cutest_test_irdis_model_process_image_dbi ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_test_irdis_model_delete",
                             cutest_test_irdis_model_delete ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
