/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "cutest_zpl_aoc_crosstalk_u.h"
#include "sph_zpl_aoc_crosstalk.h"
#include "sph_zpl_utils.h"
#include "sph_utils.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_common_keywords.h"
#include "sph_keyword_manager.h"
#include "sph_filemanager.h"
#include "sph_test_zpl_simulator.h"
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_image_tools.h"
#include <Basic.h>
#include <CUnit.h>


/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/
static
void cutest_zpl_modem_create_head_file_hwp2_plus_0( const char* infile){
	FILE* 				fp;
	char				outfilename[256];

	sprintf( outfilename, "%s.head", infile );
	fp = fopen( outfilename, "w");
	if ( fp != NULL ){
		fprintf(fp, "ESO DRS ZPL HWP2 SETTING = 22.5\n");
		fclose(fp);
	}

}

static
void cutest_zpl_modem_create_head_file_hwp2_minus_45( const char* infile){
	FILE* 				fp;
	char				outfilename[256];

	sprintf( outfilename, "%s.head", infile );
	fp = fopen( outfilename, "w");
	if ( fp != NULL ){
		fprintf(fp, "ESO DRS ZPL HWP2 SETTING = -22.5\n");
		fclose(fp);
	}

}

static
sph_keyword_manager* cutest_zpl_modem_create_cfg_file(){
	FILE* 				fp;
    sph_keyword_manager*    km      = NULL;


	fp = fopen( "sphere.cfg", "w");
    fprintf( fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWP2_SETTING.NAME = \"ESO DRS ZPL HWP2 SETTING\"\n");
    fprintf( fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWP2_SETTING.TYPE = \"DOUBLE\"\n");
    fprintf( fp, "SPH.KEYWORDS.ESO_DRS_ZPL_HWP2_SETTING.VALUE = \"0.0\"\n");
    fclose(fp);

    km = sph_keyword_manager_new();
    sph_keyword_manager_read_config( "sphere.cfg" );

    return km;
}




static int cutest_init_aoc_crosstalk_testsuite(void) {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     *--------------------------------------------------------------------*/
    sph_init_debug();
    return 0;
}

static int cutest_clean_aoc_crosstalk_testsuite(void) {
    sph_error_dump( SPH_ERROR_ERROR );
    cpl_end();
    return 0;
}



/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk only with rawframes (-U, +U)
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_1(void) {
    sph_zpl_aoc_crosstalk*     zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube, "test_aoc_crosstalk_raw_hwp2_plus_0.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 22.5, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube1, "test_aoc_crosstalk_raw_hwp2_minus_45.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, -22.5, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.6, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_minus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -200.0, 1e-10);


    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk only with rawframes (-U, +U)
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_1_1(void) {
    sph_zpl_aoc_crosstalk*     zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube, "test_aoc_crosstalk_raw_hwp2_plus_225.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 22.5, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    //rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube1, "test_aoc_crosstalk_raw_hwp2_minus_675.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, -22.5, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );

    rerr = cpl_frameset_insert( inframes, zexpcube );

    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.6, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_minus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -200.0, 1e-10);


    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}



/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk with +U rawframes: hwp2 = 22.5
  */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_2(void) {
    sph_zpl_aoc_crosstalk*     zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube, "test_aoc_crosstalk_raw_hwp2_plus_0.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 22.5, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 172.0; zero_even = 32.0;
    pi_odd = 32.0;    pi_even = 172.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube1, "test_aoc_crosstalk_raw_hwp2_minus_45.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 22.5, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);

    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.6896551724, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 406.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);


    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk with -U rawframes: -22.5
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_3(void) {
    sph_zpl_aoc_crosstalk*     zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube, "test_aoc_crosstalk_raw_hwp2_plus_0.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, -22.5, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 172.0; zero_even = 32.0;
    pi_odd = 32.0;    pi_even = 172.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube1, "test_aoc_crosstalk_raw_hwp2_minus_45.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, -22.5, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);

    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.6896551724, 1e-10);

    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk without hwp2 keyword (V)
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_4(void) {
    sph_zpl_aoc_crosstalk*   zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube, "test_aoc_crosstalk_raw_1.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 172.0; zero_even = 32.0;
    pi_odd = 32.0;    pi_even = 172.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_aoc_crosstalk_raw_2.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);

    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.6896551724, 1e-10);

    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk with rawframes & bias, dark, iff, pff and modem calibrations
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_5(void) {
    sph_zpl_aoc_crosstalk*     zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;
    cpl_frame*			 master_bias_frame		= NULL;
    cpl_frame*			 master_dark_frame		= NULL;
    cpl_frame*			 intensity_flat_frame	= NULL;
    cpl_frame*			 polarization_flat_frame	= NULL;
    cpl_frame*			 modem_eff_frame		= NULL;



    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube, "test_aoc_crosstalk_raw_hwp2_plus_0.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, 22.5, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_hwp2( &zexpcube1, "test_aoc_crosstalk_raw_hwp2_minus_45.fits.tmp", zero_odd, zero_even, pi_odd, pi_even, dV, -22.5, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );


    //simualate master bias
    zero_odd = 10; zero_even = 11; pi_odd = 12; pi_even = 14;
    master_bias_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_bias_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_bias_frame, SPH_ZPL_TAG_BIAS_CALIB );
    cpl_frame_set_group( master_bias_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, master_bias_frame );

    //simualate master dark
    zero_odd = 1; zero_even = 2; pi_odd = 3; pi_even = 5;
    master_dark_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_dark_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( master_dark_frame, SPH_ZPL_TAG_DARK_CALIB );
    cpl_frame_set_group( master_dark_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, master_dark_frame );

    //simualate intensity flat
    zero_odd = 2; zero_even = 2; pi_odd = 2; pi_even = 2;
    intensity_flat_frame = sph_test_zpl_simulator_create_quad_image_frame("test_master_intensity_flat_calib.fits.tmp", nx, ny, zero_odd, zero_even, pi_odd, pi_even);
    cpl_frame_set_tag( intensity_flat_frame, SPH_ZPL_TAG_IFF_CALIB );
    cpl_frame_set_group( intensity_flat_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, intensity_flat_frame );

    //simulate polarization flat
    polarization_flat_frame = sph_test_zpl_simulator_create_double_image_frame("test_master_polarization_flat_calib.fits.tmp", nx, ny, 200.0, 100.0 );
    cpl_frame_set_tag( polarization_flat_frame, SPH_ZPL_TAG_PFF_CALIB );
    cpl_frame_set_group( polarization_flat_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, polarization_flat_frame );

    //simulate modem efficiency
    modem_eff_frame = sph_test_zpl_simulator_create_masterframe_frame("test_master_modem_calib.fits.tmp", nx, ny, 0.5 );
    cpl_frame_set_tag( modem_eff_frame, SPH_ZPL_TAG_MODEM_EFF_CALIB );
    cpl_frame_set_group( modem_eff_frame, CPL_FRAME_GROUP_CALIB );
    //printf("master bias tag: %s\n", cpl_frame_get_tag(master_bias_frame));
    rerr = cpl_frameset_insert( inframes, modem_eff_frame );


    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);
    //CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.701754386, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 1.403508772, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 53.5/0.5, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_minus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 171.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -186.5/0.5, 1e-10);

    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief Unit test for the sph_zpl_aoc_crosstalk without hwp2 keywords but with sphere.cfg and head-files
 */
/*----------------------------------------------------------------------------*/
void cutest_zpl_aoc_crosstalk_u_run_6(void) {
    sph_zpl_aoc_crosstalk*   zpl_aoc_crosstalk  = NULL;
    cpl_parameterlist*          inparams            = NULL;
    cpl_frameset*               inframes            = NULL;
    cpl_frame*                  zexpcube            = NULL;
    cpl_frame*					zexpcube1			= NULL;
    sph_error_code              rerr                = CPL_ERROR_NONE;
    int                         nframes             = 3;
    //int                         nx                  = 100; // it must be only sync changed with sph_test_simulator_create_zplpreproc_cube_plus
    //int                         ny                  = 100; // -//-

    double              dV						= 1.0;
    //double              dS						= 0.0;

    double 				 zero_odd				= 169.0;
    double				 zero_even				= 29.0;
    double				 pi_odd					= 29.0;
    double				 pi_even				= 169.0;
    cpl_propertylist*	 pl						= NULL;

    sph_keyword_manager*        keyman      = NULL;

    cpl_error_reset();


    //create sphere.cfg
    keyman = cutest_zpl_modem_create_cfg_file();
    CU_ASSERT_PTR_NOT_NULL_FATAL( keyman );


    inparams = sph_zpl_aoc_crosstalk_create_paramlist();
    inframes = cpl_frameset_new();
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube, "test_aoc_crosstalk_raw_1.fits", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube );

    //create .head file
    cutest_zpl_modem_create_head_file_hwp2_plus_0("test_aoc_crosstalk_raw_1");

    zero_odd = 49.0; zero_even = 149.0;
    pi_odd = 149.0;    pi_even = 49.0;
    sph_test_simulator_create_zplpreproc_cube_set_filename( &zexpcube1, "test_aoc_crosstalk_raw_2.fits", zero_odd, zero_even, pi_odd, pi_even, dV, nframes );
    cpl_frame_set_tag( zexpcube1, SPH_ZPL_TAG_AOC_CROSSTALK_PREPROC_RAW );
    cpl_frame_set_group( zexpcube1, CPL_FRAME_GROUP_RAW );
    rerr = cpl_frameset_insert( inframes, zexpcube1 );
    //create .head file
    cutest_zpl_modem_create_head_file_hwp2_minus_45("test_aoc_crosstalk_raw_2");


    zpl_aoc_crosstalk = sph_zpl_aoc_crosstalk_new( inframes, inparams);
    CU_ASSERT_PTR_NOT_NULL_FATAL(zpl_aoc_crosstalk);

    rerr = sph_zpl_aoc_crosstalk_run( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEANMASTERFRAME), 0.6, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;


    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_plus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), 280.0, 1e-10);
    cpl_propertylist_delete( pl ); pl = NULL;

    pl = cpl_propertylist_load( zpl_aoc_crosstalk->aoc_crosstalk_minus_outfilename, 0);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_IFRAME), 400.0, 1e-10);
    CU_ASSERT_DOUBLE_EQUAL(cpl_propertylist_get_double(pl, SPH_COMMON_KEYWORD_QC_MEAN_DOUBLEIMAGE_PFRAME), -200.0, 1e-10);

    rerr = sph_zpl_aoc_crosstalk_delete( zpl_aoc_crosstalk );
    CU_ASSERT_EQUAL( cpl_error_get_code(), CPL_ERROR_NONE );
    sph_keyword_manager_delete();

    return;
}




/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_aoc_crosstalk recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int result       = 0;

    CU_pSuite pSuite = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("aoc_crosstalk_test: U->", cutest_init_aoc_crosstalk_testsuite,
                          cutest_clean_aoc_crosstalk_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */

    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_1: 2 cube-rawframes with +U[hwpz=22.5] and -U[hwpz=-22.5]",
                             cutest_zpl_aoc_crosstalk_u_run_1 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_1_1: 2 cube-rawframes with -U[hwpz=-22.5] and +U[hwpz=22.5]",
                             cutest_zpl_aoc_crosstalk_u_run_1_1 ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }


    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_2: 2 cube-rawframes with +U[hwpz=22.5] and +U[hwpz=22.5]",
                              cutest_zpl_aoc_crosstalk_u_run_2 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }


    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_3: 2 cube-rawframes with -U[hwpz=-22.5] and -U[hwpz=-22.5]",
                              cutest_zpl_aoc_crosstalk_u_run_3 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_4: 2 cube-rawframes without hwp2 keywords (V)",
                              cutest_zpl_aoc_crosstalk_u_run_4 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_5: 2 cube-rawframes +U and -U  with bias-, dark-, iff-, pff- and modem- calibrations",
                              cutest_zpl_aoc_crosstalk_u_run_5 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }

    if ( NULL == CU_add_test(pSuite, "sph_aoc_crosstalk_run_6: 2 cube-rawframes without hwp2 keywords, but with sphere.cfg and head-files (+U, -U)",
                              cutest_zpl_aoc_crosstalk_u_run_6 ) )
    {
        CU_cleanup_registry();
        return CU_get_error();
     }


    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
