/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_fft.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_fft.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"
#include "sph_utils.h"
#include "sph_test_image_tools.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
int cutest_init_sph_fft_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de", CPL_MSG_DEBUG);
    return 0;
}

int cutest_clean_sph_fft_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_fft_shift_image() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_GSL_MIXEDRADIX );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,10.0);
    im = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_test_image_tools_add_gauss(im,148.0,148.0,10.0,10.0);

    tim2 = sph_fft_shift_image(sphfft,tim,20.0,20.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,im);
    cpl_image_save( tim2, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_absflux( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.01 );

    cpl_image_delete(im);
    cpl_image_delete( tim2 );tim2 = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}
static
void cutest_sph_fft_shift_image_partpixel() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_GSL_MIXEDRADIX );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,10.0);
    im = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_test_image_tools_add_gauss(im,148.25,148.25,10.0,10.0);

    tim2 = sph_fft_shift_image(sphfft,tim,20.25,20.25);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,im);
    cpl_image_save( tim2, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_absflux( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.01 );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}


static
void cutest_sph_fft_shift_image_partpixel_fftw() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_FFTW3_DP );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,10.0);
    im = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_test_image_tools_add_gauss(im,148.25,148.25,10.0,10.0);

    tim2 = sph_fft_shift_image(sphfft,tim,20.25,20.25);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,im);
    cpl_image_save( tim2, "workimage_shift.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_absflux( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.01 );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_fft_shift_image_zero_shift() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_GSL_MIXEDRADIX );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,10.0);

    tim2 = sph_fft_shift_image(sphfft,tim,0.0,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,tim);
    cpl_image_save( tim2, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_absflux( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.01 );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_fft_shift_image_zero_shift_radix2() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_GSL_RADIX2 );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(128,128,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,64.0,64.0,1.0,1.0);

    tim2 = sph_fft_shift_image(sphfft,tim,0.0,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,tim);
    cpl_image_save( tim2, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_absflux( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.01 );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_fft_shift_image_zero_shift_fftw() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_FFTW3_DP );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,10.0);

    tim2 = sph_fft_shift_image(sphfft,tim,0.0,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,tim);
    cpl_image_save( tim2, "workimage_shift_zero.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_absflux( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.005 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.01 );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_fft_freq_filter() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_GSL_MIXEDRADIX );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,100000.0);
    im = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_test_image_tools_add_gauss(im,128.0,128.0,10.0,100000.0);
    sph_test_image_tools_add_noise(tim,10.0,NULL);
    tim2 = sph_fft_freq_filter(sphfft,tim,1,0.98,0.0,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_filter.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_filter.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,im);
    cpl_image_save( tim2, "workimage_filter.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_median( tim2 );
    CU_ASSERT_TRUE( val < 0.15 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.15 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.1 * cpl_image_get_max(im) );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_fft_freq_filter_fftw() {
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_FFTW3_DP );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,100000.0);
    im = sph_test_image_tools_create_flat_image_double(256,256,1.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_test_image_tools_add_gauss(im,128.0,128.0,10.0,100000.0);
    sph_test_image_tools_add_noise(tim,10.0,NULL);
    cpl_image_set(tim,100,100,10000.0);
    cpl_image_set(im,100,100,10000.0);
    tim2 = sph_fft_freq_filter(sphfft,tim,SPH_FFT_FILTER_METHOD_TH,
            0.98,0.0,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_save( tim, "workimage_filter.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_DEFAULT );
    cpl_image_save( tim2, "workimage_filter.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_subtract(tim2,im);
    cpl_image_save( tim2, "workimage_filter.fits", CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND );
    cpl_image_delete( tim );tim = NULL;
    val = cpl_image_get_median( tim2 );
    CU_ASSERT_TRUE( val < 0.15 );
    val = cpl_image_get_mean( tim2 );
    CU_ASSERT_TRUE( val < 0.15 );
    val = cpl_image_get_max( tim2 );
    CU_ASSERT_TRUE( val < 0.1 * cpl_image_get_max(im) );

    cpl_image_delete( tim2 );tim2 = NULL;
    cpl_image_delete( tim );tim = NULL;
    cpl_image_delete( im );im = NULL;
    sph_fft_delete(sphfft);sphfft=NULL;
    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
cpl_image* cutest_sph_fft_freq_filter_fftw_compare(
        sph_fft_filter_method method,
        double rms,
        double bpix_val,
        double radius,
        double param1,
        double param2)
{
    sph_fft*    sphfft            = NULL;
    cpl_image*    im                = NULL;
    cpl_image*    im2                = NULL;
    cpl_image*    tim                = NULL;
    cpl_image*    tim2                = NULL;
    double val = 0.0;

    sphfft = sph_fft_new( SPH_FFT_FFTW3_DP );

    CU_ASSERT_PTR_NOT_NULL_FATAL( sphfft );

    tim = sph_test_image_tools_create_flat_image_double(256,256,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( tim );
    sph_test_image_tools_add_gauss(tim,128.0,128.0,10.0,100000.0);
    sph_test_image_tools_add_noise(tim,rms,NULL);
    cpl_image_set(tim,100,100,bpix_val * rms);
    im = sph_test_image_tools_create_flat_image_double(256,256,0.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_test_image_tools_add_gauss(im,128.5,128.5,10.0,100000.0);
    sph_test_image_tools_add_noise(im,rms,NULL);
    cpl_image_set(im,100,100,bpix_val * rms);
    tim2 = sph_fft_freq_filter(sphfft,tim,SPH_FFT_FILTER_METHOD_FERMI,
            radius,param1,param2);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim2);
    cpl_image_delete(tim); tim = NULL;
    tim = sph_fft_shift_image(sphfft,tim2,0.5,0.5);
    cpl_image_subtract(tim,im);
    cpl_image_delete(im);
    cpl_image_delete(tim2);
    sph_fft_delete(sphfft);
    CU_ASSERT_PTR_NOT_NULL_FATAL(tim);
    return tim;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_fft_new function.
 */
/*----------------------------------------------------------------------------*/
static
void
cutest_sph_fft_freq_filter_compare_methods( ) {
    cpl_imagelist*      fermi_list = NULL;
    cpl_imagelist*      th_list = NULL;
    cpl_imagelist*      butter_list = NULL;
    int                 ii  = 0;

    fermi_list = cpl_imagelist_new();
    th_list = cpl_imagelist_new();
    butter_list = cpl_imagelist_new();

    for (ii = 0; ii < 20; ++ii) {
        cpl_imagelist_set(th_list,
                cutest_sph_fft_freq_filter_fftw_compare(
                        SPH_FFT_FILTER_METHOD_TH, 10.0,
                        10.0 * ii, 0.5, 0.0, 0.0),ii);
        cpl_imagelist_set(fermi_list,
                cutest_sph_fft_freq_filter_fftw_compare(
                        SPH_FFT_FILTER_METHOD_FERMI, 10.0,
                        10 * ii, 0.1, 0.2, 0.0),ii);
        cpl_imagelist_set(butter_list,
                cutest_sph_fft_freq_filter_fftw_compare(
                        SPH_FFT_FILTER_METHOD_BUTTER, 10.0,
                        10  * ii, 0.1, 0.05, 0.3),ii);
    }
    cpl_imagelist_save(fermi_list,"fermi_list.fits",CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE);
    cpl_imagelist_save(butter_list,"butter_list.fits",CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE);
    cpl_imagelist_save(th_list,"th_list.fits",CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE);
    cpl_imagelist_delete(fermi_list); fermi_list = NULL;
    cpl_imagelist_delete(butter_list); butter_list = NULL;
    cpl_imagelist_delete(th_list); th_list = NULL;

}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {

    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("sph_fft_shift_test", cutest_init_sph_fft_testsuite,
            cutest_clean_sph_fft_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_fft_shift",
                             cutest_sph_fft_shift_image ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_fft_freq_filter_fftw",
                             cutest_sph_fft_freq_filter_fftw ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_fft_freq_filter_compare",
                             cutest_sph_fft_freq_filter_compare_methods ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_fft_freq_filter",
                             cutest_sph_fft_freq_filter ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_fft_shift_partpixel",
                             cutest_sph_fft_shift_image_partpixel ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    if ( NULL == CU_add_test(pSuite, "sph_fft_shift_partpixel_fftw",
                             cutest_sph_fft_shift_image_partpixel_fftw ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_fft_shift_zero",
                             cutest_sph_fft_shift_image_zero_shift) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_fft_shift_zero_radix2",
                             cutest_sph_fft_shift_image_zero_shift_radix2) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_fft_shift_zero_fftw",
                             cutest_sph_fft_shift_image_zero_shift_fftw) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
