/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_framelist.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and 
 *                                  associated functions.               
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
static
int cutest_init_framelist_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    
    #if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 0, 0)
        cpl_init(CPL_INIT_DEFAULT);
    #else
        cpl_init();
    #endif
    
    cpl_msg_set_time_on();
    cpl_msg_set_component_on();
    cpl_msg_set_domain("Testing");
    cpl_msg_set_domain_on();
    cpl_msg_set_level(CPL_MSG_OFF);
    cpl_msg_set_log_level(CPL_MSG_DEBUG);

    sph_error_new();
    return 0;
}

static
int cutest_clean_framelist_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    cpl_end(); 
    return 0;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framelist_new function. 
 */
/*----------------------------------------------------------------------------*/
static
void cutest_framelist_new() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framelist_create function. 
 */
/*----------------------------------------------------------------------------*/
static
void cutest_framelist_create() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framelist_create_cpl_image function. 
 */
/*----------------------------------------------------------------------------*/
static
void cutest_framelist_create_cpl_image() {
    CU_FAIL( "Not yet implemented!");
    return;   
}


/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framelist_create_from_dit function. 
 */
/*----------------------------------------------------------------------------*/
static
void cutest_framelist_create_from_dit() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framelist_create_from_mean function. 
 */
/*----------------------------------------------------------------------------*/
static
void cutest_framelist_create_from_mean() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_framelist_delete function. 
 */
/*----------------------------------------------------------------------------*/
static
void cutest_framelist_delete() {
    CU_FAIL( "Not yet implemented!");
    return;   
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("framelist_test", cutest_init_framelist_testsuite, 
                          cutest_clean_framelist_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_framelist_new", 
                             cutest_framelist_new ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_framelist_create_cpl_image", 
                             cutest_framelist_create_cpl_image ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_framelist_create", 
                             cutest_framelist_create ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_framelist_create_from_dit", 
                             cutest_framelist_create_from_dit ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_framelist_create_from_mean", 
                             cutest_framelist_create_from_mean ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_framelist_delete", 
                             cutest_framelist_delete ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
