/* $Id: $
 *
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <cpl.h>

#include "sph_line.h"
#include "sph_common_keywords.h"
#include <math.h>
#include "sph_error.h"
#include "sph_test.h"
#include <Basic.h>
#include <CUnit.h>
#include "sph_test.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup techcal_master_test  Unit test of techcal_master recipe and
 *                                  associated functions.
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/
int cutest_init_line_testsuite() {
    return sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
}

int cutest_clean_line_testsuite() {
    sph_error_dump(0);
    return sph_end_test();
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_line_new function.
 */
/*----------------------------------------------------------------------------*/
void cutest_line_new() {
    sph_line*    line            = NULL;

    line = sph_line_new( 0.0, 0.0, 1.0, 1.0 );

    CU_ASSERT_PTR_NOT_NULL_FATAL( line );

    sph_line_delete(line); line = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_line_intersect_test function.
 */
/*----------------------------------------------------------------------------*/
void cutest_line_intersect_test() {
    sph_line*    lineA            = NULL;
    sph_line*    lineB            = NULL;
    sph_line*    lineC            = NULL;
    sph_line*    lineD            = NULL;
    sph_line*    lineE            = NULL;
    sph_line*    lineF            = NULL;
    int            result            = 0;

    lineA = sph_line_new( 0.0, 0.0, 1.0, 1.0 );
    lineB = sph_line_new( 0.0, 1.0, 1.0, 0.0 );
    lineC = sph_line_new( 1.0, 0.0, 2.0, 1.0 );
    lineD = sph_line_new( 0.0, 10.0, 10.0, 0.0 );
    lineE = sph_line_new( 10.0, 0.0, 20.0, 0.0 );
    lineF = sph_line_new( 20.0, 0.0, 30.0, 0.0 );

    CU_ASSERT_PTR_NOT_NULL_FATAL( lineA );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineB );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineC );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineD );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineE );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineF );

    result = sph_line_intersect_test( lineA, lineB );
    CU_ASSERT_EQUAL( result, 1 );
    result = sph_line_intersect_test( lineA, lineC );
    CU_ASSERT_EQUAL( result, 0 );
    result = sph_line_intersect_test( lineA, lineD );
    CU_ASSERT_EQUAL( result, 0 );
    result = sph_line_intersect_test( lineE, lineD );
    CU_ASSERT_EQUAL( result, 1 );
    result = sph_line_intersect_test( lineD, lineE );
    CU_ASSERT_EQUAL( result, 1 );
    result = sph_line_intersect_test( lineF, lineE );
    CU_ASSERT_EQUAL( result, 1 );
    result = sph_line_intersect_test( lineE, lineF );
    CU_ASSERT_EQUAL( result, 1 );
    result = sph_line_intersect_test( lineE, lineE );
    CU_ASSERT_EQUAL( result, 1 );
    sph_line_delete(lineA); lineA = NULL;
    sph_line_delete(lineB); lineB = NULL;
    sph_line_delete(lineC); lineC = NULL;
    sph_line_delete(lineD); lineD = NULL;
    sph_line_delete(lineE); lineE = NULL;
    sph_line_delete(lineF); lineF = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_line_intersect_point function.
 */
/*----------------------------------------------------------------------------*/
void cutest_line_intersect_point() {
    sph_line*    lineA            = NULL;
    sph_line*    lineB            = NULL;
    sph_line*    lineC            = NULL;
    sph_line*    lineD            = NULL;
    sph_line*    lineE            = NULL;
    sph_line*    lineF            = NULL;
    sph_point*        result            = NULL;

    lineA = sph_line_new( 0.0, 0.0, 1.0, 1.0 );
    lineB = sph_line_new( 0.0, 1.0, 1.0, 0.0 );
    lineC = sph_line_new( 1.0, 0.0, 2.0, 1.0 );
    lineD = sph_line_new( 0.0, -10.0, -10.0, 0.0 );
    lineE = sph_line_new( -10.0, 0.0, -20.0, 0.0 );
    lineF = sph_line_new( -20.0, 0.0, -30.0, 0.0 );

    CU_ASSERT_PTR_NOT_NULL_FATAL( lineA );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineB );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineC );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineD );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineE );
    CU_ASSERT_PTR_NOT_NULL_FATAL( lineF );
    result = sph_point_new(1.0,1.0);
    CU_ASSERT_EQUAL( sph_line_intersect_point( lineA, lineB, result ), CPL_ERROR_NONE);
    CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    CU_ASSERT_DOUBLE_EQUAL( result->x, 0.5, 1.0e-10 );
    CU_ASSERT_DOUBLE_EQUAL( result->y, 0.5, 1.0e-10 );
    CU_ASSERT_EQUAL( sph_line_intersect_point( lineA, lineD, result ), CPL_ERROR_NONE);
    CU_ASSERT_DOUBLE_EQUAL( result->x, -5.0, 1.0e-10 );
    CU_ASSERT_DOUBLE_EQUAL( result->y, -5.0, 1.0e-10 );
    CU_ASSERT_EQUAL( sph_line_intersect_point( lineE, lineD, result ), CPL_ERROR_NONE);
    CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    CU_ASSERT_DOUBLE_EQUAL( result->x, -10.0, 1.0e-10 );
    CU_ASSERT_DOUBLE_EQUAL( result->y, 0.0, 1.0e-10 );
    CU_ASSERT_EQUAL( sph_line_intersect_point( lineD, lineE, result ), CPL_ERROR_NONE);
    CU_ASSERT_PTR_NOT_NULL_FATAL( result );
    CU_ASSERT_DOUBLE_EQUAL( result->x, -10.0, 1.0e-10 );
    CU_ASSERT_DOUBLE_EQUAL( result->y, 0.0, 1.0e-10 );
    CU_ASSERT_EQUAL( sph_line_intersect_point( lineE, lineF, result ), -1);
    CU_ASSERT_EQUAL( sph_line_intersect_point( lineE, lineE, result ), -1);
    sph_point_delete(result); result = NULL;
    sph_line_delete(lineA); lineA = NULL;
    sph_line_delete(lineB); lineB = NULL;
    sph_line_delete(lineC); lineC = NULL;
    sph_line_delete(lineD); lineD = NULL;
    sph_line_delete(lineE); lineE = NULL;
    sph_line_delete(lineF); lineF = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test for the sph_line_delete function.
 */
/*----------------------------------------------------------------------------*/
void cutest_line_delete() {
    sph_line*    line            = NULL;

    line = sph_line_new( 0.0, 0.0, 1.0, 1.0 );

    CU_ASSERT_PTR_NOT_NULL_FATAL( line );

    sph_line_delete( line );
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit tests of techcal_master_dark recipe and associated functions
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {

    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("line_test", cutest_init_line_testsuite,
                          cutest_clean_line_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "sph_line_new",
                             cutest_line_new ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_line_intersect_test",
                             cutest_line_intersect_test ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_line_intersect_point",
                             cutest_line_intersect_point ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "sph_line_delete",
                             cutest_line_delete ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();
    return result;
}

/**@}*/
