/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_point_pattern.h"
#include "sph_utils.h"
#include <gsl/gsl_rng.h>
#include "sph_test_image_tools.h"
#include "sph_fitting.h"
#include "cutest_sph_point_pattern.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("Point pattern modules tests", cutest_init_testsuite,
                          cutest_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "Testing default constructor",
                             cutest_sph_point_pattern_new ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing centralise tiny shift bug",
                             cutest_sph_point_pattern_centralise_tiny_shifts_bug ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing default constructor from table",
                             cutest_sph_point_pattern_new_from_table ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing constructor noisy image",
                             cutest_sph_point_pattern_new_from_image_noisy ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing constructor with real image",
                             cutest_sph_point_pattern_new_from_image ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing find_closest",
                             cutest_sph_point_pattern_find_closest ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing centralise",
                             cutest_sph_point_pattern_centralise ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing finding closest neighbour",
                             cutest_sph_point_pattern_find_closest_neighbour ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing crate_image",
                             cutest_sph_point_pattern_create_image ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing crate_image accurate",
                             cutest_sph_point_pattern_create_image_accurate ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing crate_image is regular",
                             cutest_sph_point_pattern_create_image_regular ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing load ascii",
                             cutest_sph_point_pattern_load_ascii ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing rotation",
                             cutest_sph_point_pattern_rotate ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "Testing scaling",
                             cutest_sph_point_pattern_scale ) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }

    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    sph_filemanager_delete(1);
    CU_cleanup_registry();
    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite() {
    //sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_new() {
    sph_point_pattern* pp = NULL;
    pp = sph_point_pattern_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    sph_point_pattern_delete( pp );
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_new_from_table() {
    sph_point_pattern* pp = __cutest_sph_point_patter_create_pp(50);
    cpl_frame* frame = __cutest_sph_point_pattern_helper_create_table( pp, "testtab.fits");

    sph_point_pattern_delete( pp );

    sph_filemanager_add_tmp_file( cpl_frame_get_filename(frame));
    pp = sph_point_pattern_new_from_table(frame,1);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp),50);
    sph_point_pattern_delete( pp );
    cpl_frame_delete(frame);
    sph_filemanager_clean();
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_rotate() {
    sph_point_pattern* pp = NULL;

    pp = sph_point_pattern_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    sph_point_pattern_add_point(pp,0.0,0.0);
    sph_point_pattern_add_point(pp,1.0,1.0);
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp),2);

    CU_ASSERT_EQUAL( sph_point_pattern_rotate(pp, 0.0, 0.0, 45.0),
            CPL_ERROR_NONE);

    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_x,0), 0.0, 0.000000001);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_y,0), 0.0, 0.000000001);

    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_x,1), 0.0, 0.000000001);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_y,1), sqrt(1.+1.), 0.000000001);

    sph_point_pattern_save_ascii(pp,"pprot.txt");
    sph_point_pattern_delete( pp );
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_scale() {
    sph_point_pattern* pp = NULL;

    pp = sph_point_pattern_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    sph_point_pattern_add_point(pp,0.0,0.0);
    sph_point_pattern_add_point(pp,1.0,1.0);
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp),2);

    CU_ASSERT_EQUAL( sph_point_pattern_scale_around(pp, 0.0, 0.0, 2.0),
            CPL_ERROR_NONE);

    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_x,0), 0.0, 0.000000001);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_y,0), 0.0, 0.000000001);

    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_x,1), 2.0, 0.000000001);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_vector_get(pp->points_y,1), 2.0, 0.000000001);

    sph_point_pattern_save_ascii(pp,"pprot.txt");
    sph_point_pattern_delete( pp );
    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_new_from_image() {
    sph_point_pattern* pp = NULL;
    cpl_image* im = __cutest_sph_point_pattern_helper_create_image(5,5,3.0);
    pp = sph_point_pattern_new_from_image(im,NULL,5.0,3.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp),25);
    sph_point_pattern_delete( pp );
    cpl_image_delete(im);

    return;
}


/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_centralise_tiny_shifts_bug() {
    sph_point_pattern* pp = NULL;
    sph_point_pattern* pp2 = NULL;
    cpl_image*          im = NULL;
    cpl_image*          testim = NULL;
    cpl_image*          imorig = NULL;
    int                 nx = 256;
    int                 ny = 256;
    double              midx = 0.0;
    double              midy = 0.0;
    sph_point*           point = NULL;
    double              doffx0 = 0.0;
    double              doffy0 = 0.0;
    cpl_polynomial* polyx = cpl_polynomial_new(2);
    cpl_polynomial* polyy = cpl_polynomial_new(2);
    sph_polynomial_distortion_model* model_no_dist =
            sph_polynomial_distortion_model_new(polyx,polyy);
    int                 np = 5;

    pp = sph_polynomial_distortion_model_create_point_pattern(model_no_dist,
            nx, ny, np);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp), np * np);

    im = sph_point_pattern_create_image(pp,nx,ny,3.0);
    sph_point_pattern_delete(pp);
    pp = sph_point_pattern_new_from_image(im,NULL,5.0,3.0);
    cpl_image_delete(im);

    im = sph_point_pattern_create_image(pp, nx, ny, 6.0);

    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    cpl_image_save(im,"im_in.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_CREATE);
    sph_point_pattern_centralise(pp);

    pp2 = sph_point_pattern_new_from_image(im,NULL,5.0,3.0);


    midx = cpl_image_get_size_x(im) / 2.0;
    midy = cpl_image_get_size_y(im) / 2.0;

    point = sph_point_pattern_find_closest(
            pp2,
            midx,
            midy);

    doffx0 = point->x;
    doffy0 = point->y;

    SPH_ERROR_RAISE_INFO(SPH_ERROR_ERROR,
            "Offset: %6.3f, %6.3f",
            doffx0,doffy0);

    sph_point_pattern_add_offset(
            pp,
            doffx0,
            doffy0);

    testim = sph_point_pattern_create_image(
            pp,
            cpl_image_get_size_x(im),
            cpl_image_get_size_y(im),
            6.0);

    cpl_image_save(testim,"im_out.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_CREATE);

    cpl_image_subtract(im,testim);
    CU_ASSERT_DOUBLE_EQUAL( cpl_image_get_absflux(im), 0.0, 0.01);
    sph_point_pattern_delete( pp );
    sph_point_pattern_delete( pp2 );
    sph_polynomial_distortion_model_delete( model_no_dist);
    cpl_image_delete(im);
    cpl_image_delete(imorig);
    cpl_image_delete(testim);
    cpl_polynomial_delete( polyx );
    cpl_polynomial_delete( polyy );
    sph_point_delete(point); point = NULL;

    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_new_from_image_noisy() {
    sph_point_pattern* pp = NULL;
    sph_point_pattern* ppnoise = NULL;
    cpl_image* im = __cutest_sph_point_pattern_helper_create_image(6,6,4.5);
    gsl_rng*    pRNG = gsl_rng_alloc(gsl_rng_taus);
    int            ii = 0;

    cpl_image_save(im,
            "testimA.fits",
            CPL_TYPE_DOUBLE,
            NULL,
            CPL_IO_CREATE);
    pp = sph_point_pattern_new_from_image(
            im,
            NULL,
            5.0,
            4.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp),36);
    sph_test_image_tools_add_noise(im,0.0001,pRNG);
    cpl_image_save(im,"testimAnoise.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_CREATE);
    ppnoise = sph_point_pattern_new_from_image(im,NULL,5.0,4.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( ppnoise );
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(ppnoise),36);
    for (ii = 0; ii < 36; ++ii) {
        CU_ASSERT_DOUBLE_EQUAL(
                cpl_vector_get(pp->points_x,ii)-
                cpl_vector_get(ppnoise->points_x,ii),
                0.0,0.1);
        CU_ASSERT_DOUBLE_EQUAL(
                cpl_vector_get(pp->points_y,ii)-
                cpl_vector_get(ppnoise->points_y,ii),
                0.0,0.1);
    }
    cpl_image_delete(im);im = NULL;
    im = sph_point_pattern_create_image(ppnoise,256,256,1.5);
    cpl_image_save(im,"testimB.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_CREATE);
    sph_point_pattern_delete( pp );
    pp = sph_point_pattern_new_from_image(im,NULL,5.0,3.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL( sph_point_pattern_get_size(pp),36);
    cpl_image_delete(im);im = NULL;
    im = sph_point_pattern_create_image(pp,256,256,1.5);
    cpl_image_save(im,"testimC.fits",CPL_TYPE_DOUBLE,NULL,CPL_IO_CREATE);
    sph_point_pattern_delete( pp );
    sph_point_pattern_delete( ppnoise );
    cpl_image_delete(im);
    gsl_rng_free(pRNG);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_find_closest() {
    sph_point_pattern* pp = NULL;
    sph_point* p = NULL;
    pp = __cutest_sph_point_patter_create_pp(10);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    p = sph_point_pattern_find_closest( pp, 50.1, 50.1 );
    CU_ASSERT_PTR_NOT_NULL_FATAL( p );
    CU_ASSERT_DOUBLE_EQUAL( p->x, 50.0, 0.001 );
    CU_ASSERT_DOUBLE_EQUAL( p->y, 50.0, 0.001 );
    sph_point_pattern_delete( pp );
    sph_point_delete( p );
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_centralise() {
    sph_point_pattern* pp = NULL;
    sph_point* p = NULL;
    pp = __cutest_sph_point_patter_create_pp(11);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL(sph_point_pattern_centralise(pp), CPL_ERROR_NONE );
    p = sph_point_pattern_get_point(pp,5);
    CU_ASSERT_PTR_NOT_NULL_FATAL(p);
    CU_ASSERT_DOUBLE_EQUAL(p->x,0.0,0.01);
    CU_ASSERT_DOUBLE_EQUAL(p->y,0.0,0.01);
    sph_point_pattern_delete( pp );
    sph_point_delete( p );
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_load_ascii() {
    sph_point_pattern*        self = NULL;
    FILE*               f = NULL;

    f = fopen("point_pattern.txt","w");
    fprintf(f,"%s\n","# Hello this is a comment");
    fprintf(f,"%s\n","# Hello this is a comment too");
    fprintf(f,"  5.0     2.0\n");
    fprintf(f,"  6.0     3.0\n");
    fprintf(f,"  7.0     4.0\n");
    fprintf(f,"  8.0     5.0\n");
    fclose(f);
    self = sph_point_pattern_load_ascii("point_pattern.txt");
    CU_ASSERT_PTR_NOT_NULL_FATAL(self);
    CU_ASSERT_EQUAL(sph_point_pattern_get_size(self),4);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_x,0),5.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_y,0),2.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_x,1),6.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_y,1),3.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_x,2),7.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_y,2),4.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_x,3),8.0,0.00001);
    CU_ASSERT_DOUBLE_EQUAL(cpl_vector_get(self->points_y,3),5.0,0.00001);
    sph_point_pattern_delete(self); self = NULL;
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_find_closest_neighbour() {
    sph_point_pattern* pp = NULL;
    sph_point* p = NULL;
    double dist = 0.0;
    int n = 0;
    pp = __cutest_sph_point_patter_create_pp(11);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    CU_ASSERT_EQUAL(sph_point_pattern_centralise(pp), CPL_ERROR_NONE );
    n = sph_point_pattern_find_closest_neighbour(pp,6,&dist);
    CU_ASSERT_TRUE_FATAL( n > 0 );
    p = sph_point_pattern_get_point( pp, n);
    CU_ASSERT_PTR_NOT_NULL_FATAL(p);
    CU_ASSERT_DOUBLE_EQUAL(p->x,0.0,0.01);
    CU_ASSERT_DOUBLE_EQUAL(p->y,0.0,0.01);
    CU_ASSERT_DOUBLE_EQUAL(dist,14.1,0.5);
    sph_point_pattern_delete( pp );
    sph_point_delete( p );
    return;
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_create_image() {
    sph_point_pattern* pp = NULL;
    cpl_image* im = NULL;
    cpl_apertures* aps = NULL;
    pp = __cutest_sph_point_patter_create_pp(10);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    im = sph_point_pattern_create_image(pp,256,256,2.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );

    aps = cpl_apertures_extract_sigma( im, 5.0 );
    CU_ASSERT_EQUAL( cpl_apertures_get_size(aps),10);
    cpl_apertures_delete(aps);
    cpl_image_delete(im);
    sph_point_pattern_delete( pp );
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_create_image_regular() {
    sph_point_pattern* pp = NULL;
    cpl_image* im = NULL;
    cpl_image* im2 = NULL;
    pp = __cutest_sph_point_patter_create_pp(25);
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    im = sph_point_pattern_create_image(pp,256,256,2.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    sph_point_pattern_add_offset(pp,10.0,10.0);
    im2 = sph_point_pattern_create_image(pp,256,256,2.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im2 );

    cpl_image_subtract(im,im2);

    CU_ASSERT_DOUBLE_EQUAL(
            cpl_image_get_max_window(im,50,50,150,150), 0.0, 0.0001);
    CU_ASSERT_DOUBLE_EQUAL(
            cpl_image_get_min_window(im,50,50,150,150), 0.0, 0.0001);
    cpl_image_save(im,"imregular.fits", CPL_TYPE_UNSPECIFIED, NULL, CPL_IO_CREATE);
    cpl_image_delete(im);
    cpl_image_delete(im2);
    sph_point_pattern_delete( pp );
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_point_pattern_create_image_accurate() {
    sph_point_pattern* pp = NULL;
    cpl_image* im = NULL;
    cpl_image* im2 = NULL;
    cpl_image* imerr = NULL;
    double      xpos = 129.333;
    double      ypos = 128.133;
    double      xw = 3.0;
    double      yw = 3.0;

    CU_ASSERT_EQUAL_FATAL( cpl_error_get_code(), CPL_ERROR_NONE);
    pp = sph_point_pattern_new();
    CU_ASSERT_PTR_NOT_NULL_FATAL( pp );
    sph_point_pattern_add_point(pp,128.333,128.333);

    im = sph_point_pattern_create_image(pp,256,256,3.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im );
    CU_ASSERT_EQUAL_FATAL( cpl_error_get_code(), CPL_ERROR_NONE);
    sph_point_pattern_add_offset(pp,10.0,10.0);
    im2 = sph_point_pattern_create_image(pp,256,256,3.0);
    CU_ASSERT_PTR_NOT_NULL_FATAL( im2 );
    CU_ASSERT_EQUAL_FATAL( cpl_error_get_code(), CPL_ERROR_NONE);

    //sph_test_image_tools_apply_poisson_noise(im,NULL);
    imerr = cpl_image_power_create(im,0.5);
    cpl_image_add_scalar(imerr,0.01);
    cpl_image_add_scalar(im,0.1);
    sph_fitting_fit_gauss2D(im,imerr,&xpos,&ypos,&xw,&yw);
    cpl_image_save(im,"imsingle.fits", CPL_TYPE_UNSPECIFIED, NULL, CPL_IO_CREATE);
    cpl_image_fill_gaussian(im2,128.0,128.0,1000.0,4.0,4.0);
    cpl_image_save(im2,"imsingle2.fits", CPL_TYPE_UNSPECIFIED, NULL, CPL_IO_CREATE);
    SPH_ERROR_RAISE_INFO(SPH_ERROR_GENERAL,
            "Position: %6.3f, %6.3f, width: %6.3f, %6.3f", xpos, ypos, xw,yw);
    CU_ASSERT_DOUBLE_EQUAL(xpos,128.333,0.01);
    CU_ASSERT_DOUBLE_EQUAL(ypos,128.333,0.01);
    cpl_image_delete(im);
    cpl_image_delete(im2);
    cpl_image_delete(imerr);
    sph_point_pattern_delete( pp );
    SPH_RAISE_CPL_RESET
    return;
}

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/
static sph_point_pattern* __cutest_sph_point_patter_create_pp( int np ) {
    sph_point_pattern* pp = sph_point_pattern_new();
    int ii;
    for (ii = 0; ii < np; ++ii) {
        sph_point_pattern_add_point(pp, ii * 10.0, ii * 10.0 );
    }
    return pp;
}

static cpl_frame* __cutest_sph_point_pattern_helper_create_table(sph_point_pattern* pp,const char* filename) {
    cpl_frame* result = cpl_frame_new();

    cpl_frame_set_filename(result,filename);
    cpl_frame_set_tag(result,"POINT PATTERN TABLE");
    sph_point_pattern_save(pp,cpl_frame_get_filename(result));
    return result;
}

static cpl_image* __cutest_sph_point_pattern_helper_create_image( int npx, int npy, double psize ) {
    cpl_image* image = cpl_image_new(256,256,CPL_TYPE_DOUBLE );
    cpl_image* smallim = NULL;
    cpl_image* extractim = NULL;

    int size0 = (int)psize * 20;
    int size = size0;
    int nx = 256;
    int ny = 256;
    int yy = 0;
    int xx = 0;
    int ddx = 0;
    int ddy = 0;
    sph_error_code rerr = CPL_ERROR_NONE;
    double xpos = 0.0;
    double ypos = 0.0;
    double xpos0 = 0.0;
    double ypos0 = 0.0;
    double stepx = (double)nx / (npx+1);
    double stepy = (double)ny / (npy+1);
    for (xx = 0; xx < npx; ++xx) {
        for (yy = 0; yy < npy; ++yy) {
            size = size0;
            xpos = stepx * ( xx + 1 );
            ypos = stepy * ( yy + 1 );
            xpos0 = xpos - size * 0.5;
            ypos0 = ypos - size * 0.5;
            if ((int)xpos0 + size >= nx) {
                size = nx - (int)xpos0;
            }
            if ((int)ypos0 + size >= ny)
                size = ny - (int)ypos0;
            if ((int)xpos0 < 0)
                xpos0 = 0;
            if ((int)ypos0 < 0)
                ypos0 = 0;
            ddx = xpos - (int)(xpos0);
            ddy = ypos - (int)(ypos0);
            smallim = cpl_image_new(size,size,CPL_TYPE_DOUBLE);
            rerr = cpl_image_fill_gaussian( smallim, ddx,
                                     ddy,
                                     1000.0,
                                     psize, psize );
            extractim = cpl_image_extract(image,(int)(xpos0) + 1, (int)(ypos0) + 1,
                                          (int)(xpos0) + size, (int)(ypos0) + size);
            rerr = cpl_image_add(extractim,smallim);
            rerr = cpl_image_copy( image, extractim, (int)(xpos0) + 1, (int)(ypos0) + 1);
            cpl_image_delete(smallim);smallim = NULL;
            cpl_image_delete(extractim);extractim = NULL;
        }
    }
    return image;
}

/**@}*/
