/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/
#include <Basic.h>
#include <CUnit.h>
#include "sph_error.h"
#include "sph_test.h"
#include "sph_test_ngc_ir_simulator.h"
#include "sph_test_pupilimage_creator.h"
#include "sph_test_image_tools.h"
#include "sph_psfcube.h"
#include "sph_utils.h"
/*----------------------------------------------------------------------------*/
/**
 * @defgroup A CUnit Test Suite -- representing a collection of testcases
 * @par Synopsis:
 * @code
 * @endcode
 * @par Desciption:
 *
 * This module provides a collection of tests for one specific, distinct
 * module or set-up. The testing code is implemented using the CUnit
 * framework.
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*----------------------------------------------------------------------------*/
/*-                            INTERNAL HELPER FUNCTIONS                          */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/**
 @brief    Function to initiailise the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_init_testsuite() {
    /*--------------------------------------------------------------------
     * -    Prepare CPL and error logging
     * -------------------------------------------------------------------*/
    sph_init_test(__FILE__,"moeller@mpia-hd.mpg.de",CPL_MSG_DEBUG);
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Function to clean the unit test suite
 */
/*----------------------------------------------------------------------------*/
static
int cutest_clean_testsuite() {
    sph_error_dump( SPH_ERROR_ERROR );
    return sph_end_test();
}
/*----------------------------------------------------------------------------*/
/**
 @brief        A test function of the testsuite
 */
/*----------------------------------------------------------------------------*/
static
void cutest_sph_psfcube_new() {
    cpl_imagelist*      imlist  = NULL;
    sph_psfcube*        psfcube = NULL;
    cpl_vector*         shiftsx = NULL;
    cpl_vector*         shiftsy = NULL;

    imlist = cpl_imagelist_new();
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 1.0), 0);
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 2.0), 1);
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 3.0), 2);

    shiftsx = cpl_vector_new(3);
    shiftsy = cpl_vector_new(3);
    cpl_vector_set(shiftsx,0,0.0);cpl_vector_set(shiftsy,0,0.0);
    cpl_vector_set(shiftsx,1,3.0);cpl_vector_set(shiftsy,1,1.0);
    cpl_vector_set(shiftsx,2,4.0);cpl_vector_set(shiftsy,2,2.0);
    psfcube = sph_psfcube_new( imlist, shiftsx, shiftsy );
    CU_ASSERT_PTR_NOT_NULL( psfcube );

    sph_psfcube_delete(psfcube);
    return;
}
static
void cutest_sph_psfcube_get() {
    cpl_imagelist*      imlist  = NULL;
    sph_psfcube*        psfcube = NULL;
    cpl_vector*         shiftsx = NULL;
    cpl_vector*         shiftsy = NULL;
    cpl_image*          im = NULL;

    imlist = cpl_imagelist_new();
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 1.0), 0);
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 2.0), 1);
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 3.0), 2);

    shiftsx = cpl_vector_new(3);
    shiftsy = cpl_vector_new(3);
    cpl_vector_set(shiftsx,0,0.0);cpl_vector_set(shiftsy,0,0.0);
    cpl_vector_set(shiftsx,1,3.0);cpl_vector_set(shiftsy,1,1.0);
    cpl_vector_set(shiftsx,2,4.0);cpl_vector_set(shiftsy,2,2.0);
    psfcube = sph_psfcube_new( imlist, shiftsx, shiftsy );
    CU_ASSERT_PTR_NOT_NULL( psfcube );
    im = sph_psfcube_get_exact(psfcube,3.0,1.0);
    CU_ASSERT_PTR_NOT_NULL( im );
    im = sph_psfcube_get_exact(psfcube,3.0,0.0);
    CU_ASSERT_PTR_NULL( im );
    cpl_error_reset();
    sph_psfcube_delete(psfcube);
    return;
}
static
void cutest_sph_psfcube_insert() {
    cpl_imagelist*      imlist  = NULL;
    sph_psfcube*        psfcube = NULL;
    cpl_vector*         shiftsx = NULL;
    cpl_vector*         shiftsy = NULL;
    sph_error_code      rerr = CPL_ERROR_NONE;
    cpl_image*          im = NULL;

    imlist = cpl_imagelist_new();
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 1.0), 0);
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 2.0), 1);
    cpl_imagelist_set(imlist, sph_test_image_tools_create_flat_image_double(32, 32, 3.0), 2);

    shiftsx = cpl_vector_new(3);
    shiftsy = cpl_vector_new(3);
    cpl_vector_set(shiftsx,0,0.0);cpl_vector_set(shiftsy,0,0.0);
    cpl_vector_set(shiftsx,1,3.0);cpl_vector_set(shiftsy,1,1.0);
    cpl_vector_set(shiftsx,2,4.0);cpl_vector_set(shiftsy,2,2.0);
    psfcube = sph_psfcube_new( imlist, shiftsx, shiftsy );
    CU_ASSERT_PTR_NOT_NULL_FATAL( psfcube );
    rerr = sph_psfcube_insert( psfcube,
            sph_test_image_tools_create_flat_image_double(32, 32, 4.0),
            1.5, 2.5);
    CU_ASSERT_EQUAL( rerr, CPL_ERROR_NONE );
    im = sph_psfcube_get_exact(psfcube,1.5,2.5);
    CU_ASSERT_PTR_NOT_NULL( im );
    sph_psfcube_delete(psfcube);
    return;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Unit test MAIN function
 */
/*----------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    int         result      = 0;
    CU_pSuite   pSuite      = NULL;

    /* initialize the CUnit test registry */
    if (CUE_SUCCESS != CU_initialize_registry())
       return CU_get_error();


    /* add a suite to the registry */
    pSuite = CU_add_suite("Testing cutest_sph_psfcube", cutest_init_testsuite,
                          cutest_clean_testsuite );
    if (NULL == pSuite) {
       CU_cleanup_registry();
       return CU_get_error();
    }


    /* add the tests to the suite */
    if ( NULL == CU_add_test(pSuite, "cutest_sph_psfcube_new",
                              cutest_sph_psfcube_new) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "cutest_sph_psfcube_get",
                              cutest_sph_psfcube_get) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    if ( NULL == CU_add_test(pSuite, "cutest_sph_psfcube_insert",
                              cutest_sph_psfcube_insert) )
    {
       CU_cleanup_registry();
       return CU_get_error();
    }
    /* Run all tests using the CUnit Basic interface */
    CU_basic_set_mode( CU_BRM_VERBOSE );
    CU_basic_run_tests();
    sph_test_create_results_file( "results.txt" );
    result = sph_test_check_result();
    CU_cleanup_registry();

    return result;
}

/**@}*/
